#include once "inc/iup.bi"
#include once "inc/iupgl.bi"
#include once "inc/openb3d.bi"

sub _init_ constructor
  IupOpen(0,0)
  IupGLCanvasOpen()
  chdir exepath & "\media\"
end sub

sub _exit_ destructor
  IupClose()
end sub

dim shared as integer listbase=0

sub DrawText2D(txt as string,x as integer=0, y as integer=0) 
  if listbase=0 then return
  dim as integer nchars = len(txt)
  if nChars<1 then return
  dim as integer w,h,fontheight
  IupGetIntInt(IupGetHandle("CANVAS"), "RASTERSIZE", @w,@h)
  fontheight = IupGetInt(IupGetHandle("CANVAS"), "FONTSIZE" )
  
  glMatrixMode(GL_PROJECTION)
  glPushMatrix()
    glLoadIdentity()
    glOrtho(0,w,h,0,-1,1)
    glMatrixMode(GL_MODELVIEW)
    glPushMatrix()
      glLoadIdentity()
      glDisable(GL_TEXTURE_2D)
      glDisable(GL_LIGHTING)
        glColor3f(1,1,1)
        glRasterPos2i(x,y+fontheight)
        glPushAttrib (GL_LIST_BIT)
          glListBase(listbase)
          glCallLists(nChars, GL_UNSIGNED_BYTE,strptr(txt))
        glPopAttrib()
      glEnable(GL_LIGHTING)
      glEnable(GL_TEXTURE_2D)
    glPopMatrix()  
  glMatrixMode(GL_PROJECTION)
  glPopMatrix()
  glMatrixMode(GL_MODELVIEW)
end sub

function canvas_resize_cb cdecl(self as Ihandle ptr, w as integer,h as integer) as integer
  CameraViewport(IupGetHandle("CAM"),0,0,w,h)
  return IUP_DEFAULT
end function

function canvas_map_cb cdecl(self as Ihandle ptr) as integer
  dim as integer w,h
  IupGetIntInt(self, "RASTERSIZE", @w,@h)
  IupGLMakeCurrent(self)
  Graphics3D(w,h)
  
  if listbase=0 then
    listbase = glGenLists(255)
    IupGLUseFont(self,0,255,listbase)
  end if  

  dim as any ptr sun = CreateLight(LIGHT_TYPE_POINT)
  IupSetHandle("SUN",sun)
  LightColor(sun,255,128,32)
  PositionEntity(sun,0,500,-100)
  
  dim as any ptr cam = CreateCamera()
  IupSetHandle("CAM",cam)
  CameraClsColor(cam,32,32,128)
  PositionEntity(cam,0,5,-15)
  
  dim as any ptr tex = LoadTexture("renata.bmp")
  dim as any ptr br  = CreateBrush() 
  BrushTexture(br,tex)
  BrushShininess br,0.5

  dim as any ptr plane = CreatePlane()
  IupSetHandle("PLANE",plane)

  dim as any ptr quad = CreateQuad()
  IupSetHandle("QUAD",quad)
  PaintMesh(quad,br)

  dim as any ptr cube = CreateCube()
  IupSetHandle("CUBE",cube)
  PaintMesh(cube,br)
  CreateShadow(cube)

  dim as any ptr sphere = CreateSphere(32)
  IupSetHandle("SPHERE",sphere)
  PaintMesh(sphere,br)
  CreateShadow(sphere)
  dim as any ptr cone = CreateCone(32)

  IupSetHandle("CONE",cone)
  PaintMesh(cone,br)
  CreateShadow(cone)

  dim as any ptr cylinder = CreateCylinder(32)
  IupSetHandle("CYLINDER",cylinder)
  PaintMesh(cylinder,br)
  CreateShadow(cylinder)
  
  MoveEntity(cube    ,-3, 5,0)
  MoveEntity(sphere  , 3, 5,0)
  MoveEntity(quad    , 0, 5,0)
  MoveEntity(cone    , 0, 8,0)
  MoveEntity(cylinder, 0, 2,0)
  
  return IUP_DEFAULT
end function  

function canvas_unmap_cb cdecl (self as IHandle ptr) as integer
  if listbase then
    glDeleteLists(listbase,255)
    listbase=0
  end if

  return IUP_DEFAULT
end function

function redraw cdecl (self as Ihandle ptr) as integer
  static as integer frames
  
  IupGLMakeCurrent(self)
  
  TurnEntity(IupGetHandle("CONE")    ,1,1,1)
  TurnEntity(IupGetHandle("CUBE")    ,1,0,1)
  TurnEntity(IupGetHandle("QUAD")    ,0,0,1)
  TurnEntity(IupGetHandle("SPHERE")  ,1,1,0)
  TurnEntity(IupGetHandle("CYLINDER"),0,1,1)
  RenderWorld()

  frames+=1

  DrawText2D("frame: " & str(frames),0,0)
  IupGLWait(1)
  IupGLSwapBuffers(self) 
  sleep(10,1)

  return IUP_DEFAULT
end function

function idle_cb cdecl () as integer
  iupRedraw(IupGetHandle("CANVAS"),0)
  sleep(10)
  return IUP_DEFAULT
end function

function CreateGLCanvas(w as integer=640,h as integer=480, _
                        bufferbits as integer = 24, _
                        depthbits  as integer = 16, _
                        stencilbits as integer = 8) as IHandle ptr
  dim as IHandle ptr glcanvas = IupGLCanvas(NULL)                      
  IupSetAttribute(glcanvas, IUP_BUFFER, IUP_DOUBLE)
  if bufferbits>0 then
    IupStoreAttribute(glcanvas, IUP_BUFFER_SIZE ,str(bufferbits))
  end if
  if depthbits>0 then
    IupStoreAttribute(glcanvas, IUP_DEPTH_SIZE,str(depthbits))
  end if
  if stencilbits>0 then
    IupStoreAttribute(glcanvas, IUP_STENCIL_SIZE, str(stencilbits))
  end if
  IupStoreAttribute(glcanvas, "RASTERSIZE", str(w) & "x" & str(h))
  return glcanvas
end function

'
' main
'
dim as Ihandle ptr canvas = CreateGLCanvas()
IupSetHandle("CANVAS",canvas)

IupSetCallback(canvas,"MAP_CB",@canvas_map_cb)
IupSetCallback(canvas,"UNMAP_CB",@canvas_unmap_cb)
IupSetCallback(canvas,"RESIZE_CB",cast(ICallback,@canvas_resize_cb))

dim as Ihandle ptr dg = IupDialog(canvas)
IupSetAttribute(dg, "TITLE", "OpenB3D")

IupSetCallback(canvas, "ACTION", @redraw)
IupSetFunction ("IDLE_ACTION", cast(Icallback,@idle_cb))
IupShow(dg)

IupMainLoop()

