#ifndef __FTGlyph_bi_
#define __FTGlyph_bi_

/'*
 * FTGLglyph is the base class for FTGL glyphs.
 *
 * It provides the interface between Freetype glyphs and their openGL renderable counterparts.
 * This is an abstract class and derived classes must implement the ftglRenderGlyph() function.
 '/
type _FTGLGlyph as any : type FTGLglyph as _FTGLglyph

type renderCallback as sub cdecl (glyph as FTGLglyph ptr, data_ as any ptr, penx as double, peny as double,mode as integer, advancex as double ptr, advancey as double ptr)
type destroyCallback as sub cdecl (glyph as FTGLglyph ptr, data_ as any ptr)

extern "c"

/'*
 * Create a custom FTGL glyph object.
 * FIXME: maybe get rid of "base" and have advanceCallback etc. functions
 *
 * @param base  The base FTGLglyph* to subclass.
 * @param data  A pointer to private data that will be passed to callbacks.
 * @param renderCallback  A rendering callback function.
 * @param destroyCallback  A callback function to be called upon destruction.
 * @return  An FTGLglyph* object.
 '/
declare function ftglCreateCustomGlyph(base_ as FTGLglyph ptr, data_ as any ptr, rcb as renderCallback, dcb as destroyCallback) as FTGLglyph ptr

/'*
 * Destroy an FTGL glyph object.
 *
 * @param glyph  An FTGLglyph* object.
 '/
declare sub ftglDestroyGlyph(glyph as FTGLglyph ptr)

/'*
 * Render a glyph at the current pen position and compute the corresponding advance.
 *
 * @param glyph  An FTGLglyph* object.
 * @param penx  The current pen's X position.
 * @param peny  The current pen's Y position.
 * @param renderMode  Render mode to display
 * @param advancex  A pointer to an double where to write the advance's X component.
 * @param advancey  A pointer to an double where to write the advance's Y component.
 '/
declare sub ftglRenderGlyph(glyph as FTGLglyph ptr, penx as double, peny as double, renderMode as integer, advancex as double ptr, advancey as double ptr)

/'*
 * Return the advance for a glyph.
 *
 * @param glyph  An FTGLglyph* object.
 * @return  The advance's X component.
 '/
declare function ftglGetGlyphAdvance(glyph as FTGLglyph ptr) as single

/'*
 * Return the bounding box for a glyph.
 *
 * @param glyph  An FTGLglyph* object.
 * @param bounds  An array of 6 float values where the bounding box's lower
 *                left near and upper right far 3D coordinates will be stored.
 '/
declare sub ftglGetGlyphBBox(glyph as FTGLglyph ptr, bounds as single ptr)

/'*
 * Query a glyph for errors.
 *
 * @param glyph  An FTGLglyph* object.
 * @return  The current error code.
 '/
declare function ftglGetGlyphError(glyph as FTGLglyph ptr) as FT_Error

end extern

#endif

