#ifndef __IM_IMAGE_BI__
#define __IM_IMAGE_BI__

' see copyright notice at the file iup.txt

#include once "im.bi"

type _imImage
  ' main parameters
  as integer width          /' Number of columns.'/
  as integer height         /' Number of lines.'/
  as integer color_space    /' Color space descriptor. See also \ref imColorSpace.'/
  as integer data_type      /' Data type descriptor. See also \ref imDataType.'/
  as integer has_alpha      /' Indicates that there is an extra channel with alpha.
                               It will not affect the secondary parameters, i.e. the number of planes will be in fact depth+1.
                               It is always 0 unless imImageAddAlpha is called, this is done in image load functions. '/

  ' secondary parameters
  as integer depth          /' Number of planes                      (ColorSpaceDepth)         '/
  as integer line_size      /' Number of bytes per line in one plane (width * DataTypeSize)    '/
  as integer plane_size     /' Number of bytes per plane.            (line_size * height)      '/
  as integer size           /' Number of bytes occupied by the image (plane_size * depth)      '/
  as integer count          /' Number of pixels                      (width * height)          '/

  ' image data
  as any ptr ptr data       /' Image data organized as a 2D matrix with several planes.
                               But plane 0 is also a pointer to the full data.
                               The remaining planes are: data[i] = data[0] + i*plane_size       '/

  ' image attributes
  as long ptr palette       /' Color palette.  Used when depth=1. Otherwise is NULL. '/
  as integer palette_count  /' The palette is always 256 colors allocated, but can have less colors used. '/

  as any ptr attrib_table   /' in fact is an imAttribTable, but we hide this here '/
end type 
type imImage as _imImage

extern "c"

_DF_ imImageCreate(w as integer, h as integer, colspace as integer, datatype as integer) as imImage ptr
_DF_ imImageInit(w as integer, h as integer, colspace as integer, datatype as integer, datatbuf as any ptr, palatte as long ptr, palettecount as integer) as imImage ptr
_DF_ imImageCreateBased(img as const imImage ptr, w as integer, h as integer, colspace as integer, datatype as integer) as imImage ptr
_DS_ imImageDestroy(img as imImage ptr)
_DS_ imImageAddAlpha(img as imImage ptr)
_DS_ imImageReshape(img as imImage ptr, w as integer, h as integer)
_DS_ imImageCopy(src_image as const imImage ptr , dst_image as imImage ptr)
_DS_ imImageCopyData(src_image as const imImage ptr , dst_image as imImage ptr)
_DF_ imImageDuplicate(img as const imImage ptr) as imImage ptr
_DF_ imImageClone(img as const imImage ptr) as imImage ptr
_DS_ imImageSetAttribute(img as const imImage ptr, attrib as const zstring ptr, datatype as integer, count as integer,data as const any ptr)
_DF_ imImageGetAttribute(img as const imImage ptr, attrib as const zstring ptr,datatype as integer ptr,count as integer ptr) as const any ptr
_DS_ imImageGetAttributeList(img as const imImage ptr, attrib as zstring ptr ptr, attribcount as integer ptr)
_DS_ imImageClear(img as imImage ptr)
_DF_ imImageIsBitmap(img as const imImage ptr) as integer
_DS_ imImageSetPalette(img as imImage ptr, palatte as long ptr, palettecount as integer)
_DS_ imImageCopyAttributes(src_image as const imImage ptr , dst_image as imImage ptr)

_DF_ imImageMatchSize(img1 as const imImage ptr, img2 as const imImage ptr) as integer
_DF_ imImageMatchColor(img1 as const imImage ptr, img2 as const imImage ptr) as integer
_DF_ imImageMatchDataType(img1 as const imImage ptr, img2 as const imImage ptr) as integer
_DF_ imImageMatchColorSpace(img1 as const imImage ptr, img2 as const imImage ptr) as integer
_DF_ imImageMatch(img1 as const imImage ptr, img2 as const imImage ptr) as integer

_DS_ imImageSetBinary (img as imImage ptr)
_DS_ imImageMakeBinary(img as imImage ptr)
_DS_ imImageMakeGray  (img as imImage ptr)

_DF_ imFileLoadImage      (ifile as imFile ptr,indx as integer, error_ as integer ptr) as imImage ptr
_DS_ imFileLoadImageFrame (ifile as imFile ptr,indx as integer, img as imImage ptr, error_ as integer ptr)
_DF_ imFileLoadBitmap     (ifile as imFile ptr,indx as integer, error_ as integer ptr) as imImage ptr
_DF_ imFileLoadImageRegion(ifile as imFile ptr,indx as integer, bitmap as integer, error_ as integer ptr, xmin as integer, xmax as integer, ymin as integer, ymax as integer, w as integer, h as integer) as imImage ptr


_DS_ imFileLoadBitmapFrame(ifile as imFile ptr,indx as integer, img as imImage ptr, error_ as integer ptr)
_DF_ imFileSaveImage      (ifile as imFile ptr, img as const imImage ptr) as integer
_DF_ imFileImageLoad      (file_name as const zstring ptr,indx as integer, error_ as integer ptr) as imImage ptr
_DF_ imFileImageLoadBitmap(file_name as const zstring ptr,indx as integer, error_ as integer ptr) as imImage ptr
_DF_ imFileImageLoadRegion(file_name as const zstring ptr,indx as integer, bitmap as integer, error_ as integer ptr, xmin as integer, xmax as integer, ymin as integer, ymax as integer, w as integer, h as integer) as imImage ptr
_DF_ imFileImageSave      (file_name as const zstring ptr, format as const zstring ptr, img as const imImage ptr) as integer

end extern

/' Utility macro to draw the image in a CD library canvas.
   Works only for data_type IM_BYTE, and color spaces: IM_RGB, IM_MAP, IMGRAY and IM_BINARY.'/
#macro imcdCanvasPutImage(_canvas, _image, _x, _y, _w, _h, _xmin, _xmax, _ymin, _ymax)
  if (_image->color_space = IM_RGB) then
    if (_image->has_alpha) then
      cdCanvasPutImageRectRGBA(_canvas, _image->width, _image->height, _
                          cptr(ubyte ptr,_image->data[0]), _
                          cptr(ubyte ptr,_image->data[1]), _
                          cptr(ubyte ptr,_image->data[2]), _
                          cptr(ubyte ptr,_image->data[3]), _
                          _x, _y, _w, _h, _xmin, _xmax, _ymin, _ymax)
    else
      cdCanvasPutImageRectRGB(_canvas, _image->width, _image->height, _
                          cptr(ubyte ptr,_image->data[0]), _
                          cptr(ubyte ptr,_image->data[1]), _
                          cptr(ubyte ptr,_image->data[2]), _
                          _x, _y, _w, _h, _xmin, _xmax, _ymin, _ymax)
    end if
  else
    cdCanvasPutImageRectMap(_canvas, _image->width, _image->height, cptr(ubyte ptr,_image->data[0]), _image->palette, _x, _y, _w, _h, _xmin, _xmax, _ymin, _ymax)
  end if                    

#endmacro


#endif
