#ifndef __IM_PROCESS_LOC_BI__
#define __IM_PROCESS_LOC_BI__

' see copyright notice at the file iup.txt

#include once "im_image.bi"

extern "c"

' Image Resize
' Operations to change the image size.

' Only reduze the image size using the given decimation order.
' Supported decimation orders: 0 - zero order (mean)  1 - first order (bilinear decimation)
' Images must be of the same type. If image type is IM_MAP or IM_BINARY, must use order=0.
' Returns zero if the counter aborted.
_DF_ imProcessReduce(src_image as const imImage ptr, dst_image as imImage ptr, order as integer) as integer 

' Change the image size using the given interpolation order.
' Supported interpolation orders:
' 0 - zero order (near neighborhood) 
' 1 - first order (bilinear interpolation) 
' 3 - third order (bicubic interpolation)
' Images must be of the same type. If image type is IM_MAP or IM_BINARY, must use order=0.
' Returns zero if the counter aborted.
_DF_ imProcessResize(src_image as const imImage ptr, dst_image as imImage ptr, order as integer) as integer

' Reduze the image area by 4 (w/2,h/2).
' Images must be of the same type. Destiny image size must be source image width/2, height/2.
' Can not operate on IM_MAP nor IM_BINARY images.
_DS_ imProcessReduceBy4(src_image as const imImage ptr, dst_image as imImage ptr)

' Extract a rectangular region from an image.
' Images must be of the same type. Destiny image size must be smaller than source image width-xmin, height-ymin.
'ymin and xmin must be >0 and <size.
_DS_ imProcessCrop(src_image as const imImage ptr, dst_image as imImage ptr, xmin as integer, ymin as integer)

' Insert a rectangular region in an image.
' Images must be of the same type. Region image size can be larger than source image.
' ymin and xmin must be >0 and <size.
' Source and destiny must be of the same size. Can be done in place.
_DS_ imProcessInsert(src_image as const imImage ptr, region_image as const imImage ptr, dst_image as imImage ptr, xmin as integer, ymin as integer)

' Increase the image size by adding pixels with zero value.
' Images must be of the same type. Destiny image size must be greatter than source image width+xmin, height+ymin.
_DS_ imProcessAddMargins(src_image as const imImage ptr, dst_image as imImage ptr, xmin as integer, ymin as integer)

' Geometric Operations
' Operations to change the shape of the image.


' Calculates the size of the new image after rotation.
_DS_ imProcessCalcRotateSize(width_ as integer, height as integer, new_width as integer ptr, new_height as integer ptr, cos0 as double, sin0 as double)

' Rotates the image using the given interpolation order see imProcessResize).n
' Images must be of the same type. The destiny size can be calculated using  imProcessCalcRotateSize to fit the new image size, 
' or can be any size, including the original size. The rotation is relative to the center of the image.
' Returns zero if the counter aborted.
_DF_ imProcessRotate(src_image as const imImage ptr, dst_image as imImage ptr, cos0 as double, sin0 as double, order as integer) as integer

' Rotates the image using the given interpolation order (see imProcessResize).
' Images must be of the same type. Destiny can have any size, including the original size.
' The rotation is relative to the reference point. But the result can be shifted to the origin.
' Returns zero if the counter aborted.
_DF_ imProcessRotateRef(src_image as const imImage ptr, dst_image as imImage ptr, cos0 as double, sin0 as double, x as integer, y as integer, to_origin as integer, order as integer) as integer

' Rotates the image in 90 degrees counterclockwise or clockwise. Swap columns by lines.
' Images must be of the same type. Destiny width and height must be source height and width.
' Direction can be clockwise (1) or counter clockwise (-1).
_DS_ imProcessRotate90(src_image as const imImage ptr, dst_image as imImage ptr, dir_clockwise as integer)

' Rotates the image in 180 degrees. Swap columns and swap lines.
' Images must be of the same type and size.
_DS_ imProcessRotate180(src_image as const imImage ptr, dst_image as imImage ptr)

' Mirror the image in a horizontal flip. Swap columns.
' Images must be of the same type and size. Can be done in-place.
_DS_ imProcessMirror(src_image as const imImage ptr, dst_image as imImage ptr)

' Apply a vertical flip. Swap lines.
' Images must be of the same type and size. Can be done in-place.
_DS_ imProcessFlip(src_image as const imImage ptr, dst_image as imImage ptr)

' Apply a radial distortion using the given interpolation order (see imProcessResize).
' Images must be of the same type and size. Returns zero if the counter aborted.
_DF_ imProcessRadial(src_image as const imImage ptr, dst_image as imImage ptr, k1 as single, order as integer) as integer

' Apply a swirl distortion using the given interpolation order (see imProcessResize).
' Images must be of the same type and size. Returns zero if the counter aborted.
_DF_ imProcessSwirl(src_image as const imImage ptr, dst_image as imImage ptr, k1 as single, order as integer) as integer

' Split the image in two images, one containing the odd lines and other containing the even lines.
' Images must be of the same type. Height of the output images must be half the height of the input image.
' If the height of the input image is odd then the first image must have height equals to half+1.
_DS_ imProcessInterlaceSplit(src_image as const imImage ptr, dst_image1 as imImage ptr, dst_image as imImage ptr)

' Morphology Operations for Gray Images

' Base gray morphology convolution.
' Supports all data types except IM_CFLOAT. Can be applied on color images.
' Kernel is always IM_INT. Use kernel size odd for better results.
' Use -1 for don't care positions in kernel. Kernel values are added to image values, then
' you can use the maximum or the minimum within the kernel area.
' No border extensions are used. 
' All the gray morphology operations use this function.
' If the kernel image attribute "Description" exists it is used by the counter.
_DF_ imProcessGrayMorphConvolve(src_image as const imImage ptr, dst_image as imImage ptr, kernel as const imImage ptr, ismax as integer) as integer

' Gray morphology convolution with a kernel full of "0"s and use minimum value.
_DF_ imProcessGrayMorphErode(src_image as const imImage ptr, dst_image as imImage ptr, kernel_size as integer) as integer

' Gray morphology convolution with a kernel full of "0"s and use maximum value.
_DF_ imProcessGrayMorphDilate(src_image as const imImage ptr, dst_image as imImage ptr, kernel_size as integer) as integer

' Erode+Dilate.
_DF_ imProcessGrayMorphOpen(src_image as const imImage ptr, dst_image as imImage ptr, kernel_size as integer) as integer

' Dilate+Erode.
_DF_ imProcessGrayMorphClose(src_image as const imImage ptr, dst_image as imImage ptr, kernel_size as integer) as integer

' Open+Difference.
_DF_ imProcessGrayMorphTopHat(src_image as const imImage ptr, dst_image as imImage ptr, kernel_size as integer) as integer

' Close+Difference.
_DF_ imProcessGrayMorphWell(src_image as const imImage ptr, dst_image as imImage ptr, kernel_size as integer) as integer

' Difference(Erode, Dilate).
_DF_ imProcessGrayMorphGradient(src_image as const imImage ptr, dst_image as imImage ptr, kernel_size as integer) as integer



' Morphology Operations for Binary Images

' Base binary morphology convolution.
' Images are all IM_BINARY. Kernel is IM_INT, but values can be only 1, 0 or -1. Use kernel size odd for better results.
' Hit white means hit=1 and miss=0, or else hit=0 and miss=1. 
' Use -1 for don't care positions in kernel. Kernel values are simply compared with image values. 
' The operation can be repeated by a number of iterations. 
' The border is zero extended. 
' Almost all the binary morphology operations use this function.
' If the kernel image attribute "Description" exists it is used by the counter.
_DF_ imProcessBinMorphConvolve(src_image as const imImage ptr, dst_image as imImage ptr, kernel as const imImage ptr, hit_white as integer, iter as integer) as integer

' Binary morphology convolution with a kernel full of "1"s and hit white.
_DF_ imProcessBinMorphErode(src_image as const imImage ptr, dst_image as imImage ptr, kernel_size as integer, iter as integer) as integer

' Binary morphology convolution with a kernel full of "0"s and hit black.
_DF_ imProcessBinMorphDilate(src_image as const imImage ptr, dst_image as imImage ptr, kernel_size as integer, iter as integer) as integer

' Erode+Dilate.
_DF_ imProcessBinMorphOpen(src_image as const imImage ptr, dst_image as imImage ptr, kernel_size as integer, iter as integer) as integer

' Dilate+Erode.
_DF_ imProcessBinMorphClose(src_image as const imImage ptr, dst_image as imImage ptr, kernel_size as integer, iter as integer) as integer

' Erode+Difference.
_DF_ imProcessBinMorphOutline(src_image as const imImage ptr, dst_image as imImage ptr, kernel_size as integer, iter as integer) as integer

' Thins the supplied binary image using Rosenfeld's parallel thinning algorithm.
_DS_ imProcessBinMorphThin(src_image as const imImage ptr, dst_image as imImage ptr)

' Rank Convolution Operations
' All the rank convolution use the same base function. Near the border the base function 
' includes only the real image pixels in the rank. No border extensions are used.

' Rank convolution using the median value.
' Returns zero if the counter aborted.
' Supports all data types except IM_CFLOAT. Can be applied on color images.
_DF_ imProcessMedianConvolve(src_image as const imImage ptr, dst_image as imImage ptr, kernel_size as integer) as integer

' Rank convolution using (maximum-minimum) value.
' Returns zero if the counter aborted.
' Supports all data types except IM_CFLOAT. Can be applied on color images.
_DF_ imProcessRangeConvolve(src_image as const imImage ptr, dst_image as imImage ptr, kernel_size as integer) as integer

' Rank convolution using the closest maximum or minimum value.
' Returns zero if the counter aborted.
' Supports all data types except IM_CFLOAT. Can be applied on color images.
_DF_ imProcessRankClosestConvolve(src_image as const imImage ptr, dst_image as imImage ptr, kernel_size as integer) as integer

' Rank convolution using the maximum value.
' Returns zero if the counter aborted.
' Supports all data types except IM_CFLOAT. Can be applied on color images.
_DF_ imProcessRankMaxConvolve(src_image as const imImage ptr, dst_image as imImage ptr, kernel_size as integer) as integer

' Rank convolution using the minimum value.
' Returns zero if the counter aborted.
' Supports all data types except IM_CFLOAT. Can be applied on color images.
_DF_ imProcessRankMinConvolve(src_image as const imImage ptr, dst_image as imImage ptr, kernel_size as integer) as integer

' Threshold using a rank convolution with a range contrast function.
' Supports all integer IM_GRAY images as source, and IM_BINARY as destiny.
' Local variable threshold by the method of Bernsen. \n
' Returns zero if the counter aborted.
_DF_ imProcessRangeContrastThreshold(src_image as const imImage ptr, dst_image as imImage ptr, kernel_size as integer, min_range as integer) as integer

' Threshold using a rank convolution with a local max function.
' Returns zero if the counter aborted.
' Supports all integer IM_GRAY images as source, and IM_BINARY as destiny.
_DF_ imProcessLocalMaxThreshold(src_image as const imImage ptr, dst_image as imImage ptr, kernel_size as integer, min_level as integer) as integer

' Convolution Operations

' Base Convolution with a kernel.
' Kernel can be IM_INT or IM_FLOAT, but always IM_GRAY. Use kernel size odd for better results.
' Supports all data types. The border is mirrored.
' Returns zero if the counter aborted. Most of the convolutions use this function.\n
' If the kernel image attribute "Description" exists it is used by the counter.
_DF_ imProcessConvolve(src_image as const imImage ptr, dst_image as imImage ptr, kernel as const imImage ptr) as integer

' Base convolution when the kernel is separable. Only the first line and the first column will be used.
' Returns zero if the counter aborted.
' If the kernel image attribute "Description" exists it is used by the counter.
_DF_ imProcessConvolveSep(src_image as const imImage ptr, dst_image as imImage ptr, kernel as const imImage ptr) as integer

' Base Convolution with two kernels. The result is the magnitude of the result of each convolution.
' Kernel can be IM_INT or IM_FLOAT, but always IM_GRAY. Use kernel size odd for better results.
' Supports all data types. The border is mirrored.
' Returns zero if the counter aborted. Most of the convolutions use this function.
' If the kernel image attribute "Description" exists it is used by the counter.
_DF_ imProcessConvolveDual(src_image as const imImage ptr, dst_image as imImage ptr, kernel1 as const imImage ptr, kernel2 as const imImage ptr) as integer

' Repeats the convolution a number of times.
' Returns zero if the counter aborted.
' If the kernel image attribute "Description" exists it is used by the counter.
_DF_ imProcessConvolveRep(src_image as const imImage ptr, dst_image as imImage ptr, kernel as const imImage ptr, count as integer) as integer

' Convolve with a kernel rotating it 8 times and getting the absolute maximum value.
' Kernel must be square.
' The rotation is implemented only for kernel sizes 3x3, 5x5 and 7x7.
' Supports all data types except IM_CFLOAT.
' Returns zero if the counter aborted.
' If the kernel image attribute "Description" exists it is used by the counter.
_DF_ imProcessCompassConvolve(src_image as const imImage ptr, dst_image as imImage ptr, kernel as imImage ptr) as integer

' Utility function to rotate a kernel one time.
_DS_ imProcessRotateKernel(kernel as imImage ptr)

' Difference(Gaussian1, Gaussian2).
' Supports all data types, 
' but if source is IM_BYTE or IM_USHORT destiny image must be of type IM_INT.
_DF_ imProcessDiffOfGaussianConvolve(src_image as const imImage ptr, dst_image as imImage ptr, stddev1 as single, stddev2 as single) as integer

' Convolution with a laplacian of a gaussian kernel.
' Supports all data types, 
' but if source is IM_BYTE or IM_USHORT destiny image must be of type IM_INT.
_DF_ imProcessLapOfGaussianConvolve(src_image as const imImage ptr, dst_image as imImage ptr, stddev as single) as integer

' Convolution with a kernel full of "1"s inside a circle.
' Supports all data types.
_DF_ imProcessMeanConvolve(src_image as const imImage ptr, dst_image as imImage ptr, kernel_size as integer) as integer

' Convolution with a float gaussian kernel.n
' If sdtdev is negative its magnitude will be used as the kernel size.
' Supports all data types.
_DF_ imProcessGaussianConvolve(src_image as const imImage ptr, dst_image as imImage ptr, stddev as single) as integer

' Convolution with a barlett kernel.
' Supports all data types.
_DF_ imProcessBarlettConvolve(src_image as const imImage ptr, dst_image as imImage ptr, kernel_size as integer) as integer

' Magnitude of the sobel convolution.
' Supports all data types.
_DF_ imProcessSobelConvolve(src_image as const imImage ptr, dst_image as imImage ptr) as integer

' Magnitude of the prewitt convolution.
' Supports all data types.
_DF_ imProcessPrewittConvolve(src_image as const imImage ptr, dst_image as imImage ptr) as integer

' Spline edge dectection.
' Supports all data types.
_DF_ imProcessSplineEdgeConvolve(src_image as const imImage ptr, dst_image as imImage ptr) as integer

' Finds the zero crossings of IM_INT and IM_FLOAT images. Crossings are marked with non zero values
' indicating the intensity of the edge. It is usually used after a second derivative, laplace.
_DS_ imProcessZeroCrossing(src_image as const imImage ptr, dst_image as imImage ptr)

' First part of the Canny edge detector. Includes the gaussian filtering and the nonmax suppression.
' After using this you could apply a Hysteresis Threshold, see imProcessHysteresisThreshold.
' Image must be IM_BYTE/IM_GRAY.
_DS_ imProcessCanny(src_image as const imImage ptr, dst_image as imImage ptr, stddev as single)

' Calculates the kernel size given the standard deviation.
' If sdtdev is negative its magnitude will be used as the kernel size.
_DF_ imGaussianStdDev2KernelSize(stddev as single) as integer

' Calculates the standard deviation given the kernel size.
_DF_ imGaussianKernelSize2StdDev(kernel_size as integer) as single

' Edge enhancement using Unsharp mask. stddev control the gaussian filter, 
' amount controls how much the edges will enhance the image (0<amount<1), and
' threshold controls which edges will be considered, it compares to twice of the absolute size of the edge.
' Although very similar to \ref imProcessSharp, produces better results.
_DF_ imProcessUnsharp(src_image as const imImage ptr, dst_image as imImage ptr, stddev as single, amount as single, threshold as single) as integer

' Edge enhancement using Laplacian8 mask. 
' amount controls how much the edges will enhance the image (0<amount<1), and
' threshold controls which edges will be considered, it compares to twice of the absolute size of the edge.
_DF_ imProcessSharp(src_image as const imImage ptr, dst_image as imImage ptr, amount as single, threshold as single) as integer

' Edge enhancement using a given kernel.
' If kernel has all positive values, then the unsharp technique is used, else sharp is used.
' amount controls how much the edges will enhance the image (0<amount<1), and
'  threshold controls which edges will be considered, it compares to twice of the absolute size of the edge.
_DF_ imProcessSharpKernel(src_image as const imImage ptr, kernel as const imImage ptr, dst_image as imImage ptr, amount as single, threshold as single) as integer


end extern


#endif ' __IM_PROCESS_LOC_BI__
