#ifndef __IM_PROCESS_PON_BI__
#define __IM_PROCESS_PON_BI__

' see copyright notice at the file iup.txt

#include once "im_image.bi"


' Render Funtion.
type imRenderFunc as function cdecl (x as integer, y as integer, d as integer, param as single ptr) as single

' Render Conditional Funtion.
type imRenderCondFunc as function cdecl (x as integer, y as integer, d as integer, cond as integer ptr, param as single ptr) as single

extern "c"

' Arithmetic Operations 
' Simple math operations for images.

' Unary Arithmetic Operations.
' Inverse and log may lead to math exceptions.

enum imUnaryOp
  IM_UN_EQL,    ' equal             =     a
  IM_UN_ABS,    ' abssolute         =    |a|
  IM_UN_LESS,   ' less              =    -a
  IM_UN_INV,    ' invert            =   1/a       (#)
  IM_UN_SQR,    ' square            =     a*a
  IM_UN_SQRT,   ' square root       =     a^(1/2)
  IM_UN_LOG,    ' natural logarithm =  ln(a)      (#)
  IM_UN_EXP,    ' exponential       = exp(a)
  IM_UN_SIN,    ' sine              = sin(a)
  IM_UN_COS,    ' cosine            = cos(a)
  IM_UN_CONJ,   ' complex conjugate =     ar - ai*i
  IM_UN_CPXNORM ' complex normalization by magnitude = a / cpxmag(a)
end enum

' Apply an arithmetic unary operation.n
' Can be done in place, images must match size.

_DS_ imProcessUnArithmeticOp(src_image as const imImage ptr, dst_image as imImage ptr, op as imUnaryOp)

' Binary Arithmetic Operations.
' Divide may lead to math exceptions.
enum imBinaryOp
  IM_BIN_ADD,    ' add         =    a+b            */
  IM_BIN_SUB,    ' subtract    =    a-b            */
  IM_BIN_MUL,    ' multiply    =    a*b            */
  IM_BIN_DIV,    ' divide      =    a/b            (#) */
  IM_BIN_DIFF,   ' difference  =    |a-b|          */
  IM_BIN_POW,    ' power       =    a^b            */
  IM_BIN_MIN,    ' minimum     =    (a < b)? a: b  */
  IM_BIN_MAX     ' maximum     =    (a > b)? a: b  */
end enum

' Apply a binary arithmetic operation.
' Can be done in place, images must match size.
' Source images must match type, destiny image can be several types depending on source:
' One exception is that you can combine complex with float resulting complex.
' If destiny is byte, then the result is cropped to 0-255.
_DS_ imProcessArithmeticOp(src_image1 as const imImage ptr,src_image2 as const imImage ptr,dst_image as imImage ptr, op as imBinaryOp)

' Apply a binary arithmetic operation with a constant value.
' Can be done in place, images must match size.
' Destiny image can be several types depending on source:
' The constant value is type casted to an apropriate type before the operation.
' If destiny is byte, then the result is cropped to 0-255.
_DS_ imProcessArithmeticConstOp(src_image as const imImage ptr, src_const as single,dst_image as imImage ptr, op as imBinaryOp)

' Blend two images using an alpha value = [a * alpha + b * (1 - alpha)].
' Can be done in place, images must match size and type. \n
' alpha value must be in the interval [0.0 - 1.0].
_DS_ imProcessBlendConst(src_image1 as const imImage ptr , src_image2 as const imImage ptr, dst_image as imImage ptr, alpha_ as single)

' Blend two images using an alpha channel = [a * alpha + b * (1 - alpha)].
' Can be done in place, images must match size and type.n
' alpha_image must have the same data type except for complex images that must be float, and color_space must be IM_GRAY.
' integer alpha values must be:
' 0 - 255 IM_BYTE , 0 - 65535 IM_USHORT, 0 - 2147483647 IM_INT
_DS_ imProcessBlend(src_image1 as const imImage ptr, src_image2 as const imImage ptr, alpha_image as const imImage ptr,dst_image as imImage ptr)

' Split a complex image into two images with real and imaginary parts
' or magnitude and phase parts (polar).
' Source image must be IM_CFLOAT, destiny images must be IM_FLOAT.
_DS_ imProcessSplitComplex(src_image as const imImage ptr, dst_image1 as imImage ptr,dst_image2 as imImage ptr, polar as integer)

' Merges two images as the real and imaginary parts of a complex image, or as magnitude and phase parts (polar = 1).
' Source images must be IM_FLOAT, destiny image must be IM_CFLOAT.
_DS_ imProcessMergeComplex(src_image1 as const imImage ptr, src_image2 as const imImage ptr, dst_image as imImage ptr, polar as integer)

' Calculates the mean of multiple images. Images must match size and type.
_DS_ imProcessMultipleMean(src_image_list as const imImage ptr ptr, src_image_count as integer, dst_image as imImage ptr)

' Calculates the standard deviation of multiple images.
' Images must match size and type. Use \ref imProcessMultipleMean to calculate the mean_image.
_DS_ imProcessMultipleStdDev(src_image_list as const imImage ptr ptr , src_image_count as integer, mean_image as const imImage ptr, dst_image as imImage ptr)

' Calculates the auto-covariance of an image with the mean of a set of images.
' Images must match size and type. Returns zero if the counter aborted. Destiny is IM_FLOAT.
_DF_ imProcessAutoCovariance(src_image as const imImage ptr, mean_image as const imImage ptr, dst_image as imImage ptr) as integer

' Multiplies the conjugate of one complex image with another complex image.
' Images must match size. Conj(img1) * img2,  Can be done in-place.
_DS_ imProcessMultiplyConj(src_image1 as const imImage ptr, src_image2 as const imImage ptr, dst_image as imImage ptr)

' Additional Image Quantization Operations
' Additionally operations to the imConvertColorSpace function.

' Converts a RGB image to a MAP image using uniform quantization 
' with an optional 8x8 ordered dither. The RGB image must have data type IM_BYTE.
_DS_ imProcessQuantizeRGBUniform(src_image as const imImage ptr,dst_image as imImage ptr, do_dither as integer)

' Quantizes a gray scale image in less that 256 grays using uniform quantization.
' Both images must be IM_BYTE/IM_GRAY. Can be done in place. 
_DS_ imProcessQuantizeGrayUniform(src_image as const imImage ptr, dst_image as imImage ptr, grays as integer)


' Histogram Based Operations


' Performs an histogram expansion based on a percentage of the number of pixels.
' Percentage defines an amount of pixels to include at the lowest level and at the highest level.
' If its is zero only empty counts of the histogram will be considered.
' Images must be IM_BYTE/(IM_RGB or IM_GRAY). Can be done in place.
' To expand the gammut without using the histogram, by just specifing the lowest and highest levels
' use the IM_GAMUT_EXPAND tone gammut operation imProcessToneGamut).
_DS_ imProcessExpandHistogram(src_image as const imImage ptr, dst_image as imImage ptr, percent as single)

' Performs an histogram equalization.
' Images must be IM_BYTE/(IM_RGB or IM_GRAY). Can be done in place. 
_DS_ imProcessEqualizeHistogram(src_image as const imImage ptr,dst_image as imImage ptr)

' Color Processing Operations

' Split a RGB image into luma and chroma.
' Chroma is calculated as R-Y,G-Y,B-Y. Source image must be IM_RGB/IM_BYTE.
' luma image is IM_GRAY/IM_BYTE and chroma is IM_RGB/IM_BYTE.
' Source and destiny must have the same size. 
_DS_ imProcessSplitYChroma(src_image as const imImage ptr, y_image as imImage ptr, chroma_image as imImage ptr)

' Split a RGB image into HSI planes.
' Source image must be IM_RGB/IM_BYTE,IM_FLOAT. Destiny images are all IM_GRAY/IM_FLOAT. 
' Source images must normalized to 0-1 if type is IM_FLOAT (\ref imProcessToneGamut can be used). See \ref hsi for a definition of the color conversion.
' Source and destiny must have the same size. 
_DS_ imProcessSplitHSI(src_image as const imImage ptr, h_image as imImage ptr, s_image as imImage ptr, i_image as imImage ptr)

' Merge HSI planes into a RGB image.
' Source images must be IM_GRAY/IM_FLOAT. Destiny image can be IM_RGB/IM_BYTE,IM_FLOAT.
' Source and destiny must have the same size. See \ref hsi for a definition of the color conversion.
_DS_ imProcessMergeHSI(h_image as const imImage ptr, s_image as const imImage ptr, i_image as const imImage ptr, dst_image as imImage ptr)

' Split a multicomponent image into separate components.
' Destiny images must be IM_GRAY. Size and data types must be all the same.
' The number of destiny images must match the depth of the source image.
_DS_ imProcessSplitComponents(src_image as const imImage ptr, dst_image_list as imImage ptr ptr)

' Merges separate components into a multicomponent image.
' Source images must be IM_GRAY. Size and data types must be all the same.
' The number of source images must match the depth of the destiny image.
_DS_ imProcessMergeComponents(src_image_list as const imImage ptr ptr, dst_image as imImage ptr)

' Normalize the color components by their sum. Example: c1 = c1/(c1+c2+c3).
' Destiny image must be IM_FLOAT. 
_DS_ imProcessNormalizeComponents(src_image as const imImage ptr, dst_image as imImage ptr)

' Replaces the source color by the destiny color.
' The color will be type casted to the image data type.
' The colors must have the same number of components of the images.
' Supports all color spaces and all data types except IM_CFLOAT.
_DS_ imProcessReplaceColor(src_image as const imImage ptr, dst_image as imImage ptr, src_color as single ptr, dst_color as single ptr)

' Logical Arithmetic Operations 
enum imLogicOp
  IM_BIT_AND,   ' and  =   a & b
  IM_BIT_OR,    ' or   =   a | b
  IM_BIT_XOR    ' xor  = ~(a | b)
end enum

' Apply a logical operation.
' Images must have data type IM_BYTE, IM_USHORT or IM_INT. Can be done in place. 
_DS_ imProcessBitwiseOp(src_image1 as const imImage ptr,src_image2 as const imImage ptr, dst_image as imImage ptr, op as imLogicOp)

' Apply a logical NOT operation.
' Images must have data type IM_BYTE, IM_USHORT or IM_INT. Can be done in place. 
_DS_ imProcessBitwiseNot(src_image as const imImage ptr, dst_image as imImage ptr)

' Apply a bit mask.
' The same as imProcessBitwiseOp but the second image is replaced by a fixed mask.
' Images must have data type IM_BYTE. It is valid only for AND, OR and XOR. Can be done in place.
_DS_ imProcessBitMask(src_image as const imImage ptr, dst_image as imImage ptr, mask as ubyte, op as imLogicOp)

' Extract or Reset a bit plane. For ex: 000X0000 or XXX0XXXX (plane=3).
' Images must have data type IM_BYTE. Can be done in place. 
_DS_ imProcessBitPlane(src_image as const imImage ptr, dst_image as imImage ptr,  plane as integer, do_reset as integer)


' Synthetic Image Render
' Renders some 2D mathematical functions as images. All the functions operates in place 
' and supports all data types except IM_CFLOAT.

' Render a synthetic image using a render function.
' plus will make the render be added to the current image data, 
' or else all data will be replaced. All the render functions use this or the conditional function.
' Returns zero if the counter aborted.
_DF_ imProcessRenderOp(img as imImage ptr, render_func as imRenderFunc, render_name as zstring ptr, param as single ptr, plus as integer) as integer

' Render a synthetic image using a conditional render function.
' Data will be rendered only if the condional param is true.
' Returns zero if the counter aborted.
_DF_ imProcessRenderCondOp(img as imImage ptr, render_cond_func as imRenderCondFunc, render_name as zstring ptr, param as single ptr) as integer

' Render speckle noise on existing data. Can be done in place.
_DF_ imProcessRenderAddSpeckleNoise(src_image as const imImage ptr, dst_image as imImage ptr, percent as single) as integer

' Render gaussian noise on existing data. Can be done in place.
_DF_ imProcessRenderAddGaussianNoise(src_image as const imImage ptr, dst_image as imImage ptr, mean as single,  stddev as single) as integer

' Render uniform noise on existing data. Can be done in place.
_DF_ imProcessRenderAddUniformNoise(src_image as const imImage ptr, dst_image as imImage ptr, mean as single, stddev as single) as integer

' Render random noise.
_DF_ imProcessRenderRandomNoise(img as imImage ptr) as integer

' Render a constant. The number of values must match the depth of the image.
_DF_ imProcessRenderConstant(img as imImage ptr, value as single ptr) as integer

' Render a centered wheel.
_DF_ imProcessRenderWheel(img as imImage ptr, internal_radius as integer, external_radius as integer) as integer

' Render a centered cone.
_DF_ imProcessRenderCone(img as imImage ptr, radius as integer) as integer

' Render a centered tent.
_DF_ imProcessRenderTent(img as imImage ptr, tent_width as integer, tent_height as integer) as integer

' Render a ramp. Direction can be vertical (1) or horizontal (0).
_DF_ imProcessRenderRamp(img as imImage ptr, start as integer, end_ as integer, vert_dir as integer) as integer

' Render a centered box.
_DF_ imProcessRenderBox(img as imImage ptr, box_width as integer, box_height as integer) as integer

' Render a centered sinc.
_DF_ imProcessRenderSinc(img as imImage ptr, x_period as single, y_period as single) as integer

' Render a centered gaussian.
_DF_ imProcessRenderGaussian(img as imImage ptr, stddev as single) as integer

' Render the laplacian of a centered gaussian.
_DF_ imProcessRenderLapOfGaussian(img as imImage ptr, stddev as single) as integer

' Render a centered cosine.
_DF_ imProcessRenderCosine(img as imImage ptr, x_period as single, y_period as single) as integer 

' Render a centered grid.
_DF_ imProcessRenderGrid(img as imImage ptr, x_space as integer, y_space as integer) as integer

' Render a centered chessboard.
_DF_ imProcessRenderChessboard(img as imImage ptr, x_space as integer, y_space as integer) as integer


' Tone Gamut Operations
' Operations that try to preserve the min-max interval in the output (the dynamic range).

' Tone Gamut Operations.
enum imToneGamut
  IM_GAMUT_NORMALIZE, ' normalize = (a-min) / (max-min)     (destiny image must be IM_FLOAT)
  IM_GAMUT_POW,       ' pow       = ((a-min) / (max-min))^gamma * (max-min) + min param[0]=gamma
  IM_GAMUT_LOG,       ' log       = log(K * (a-min) / (max-min) + 1))*(max-min)/log(K+1) + min param[0]=K     (K>0)
  IM_GAMUT_EXP,       ' exp       = (exp(K * (a-min) / (max-min)) - 1))*(max-min)/(exp(K)-1) + min param[0]=K
  IM_GAMUT_INVERT,    ' invert    = max - (a-min)
  IM_GAMUT_ZEROSTART, ' zerostart = a - min
  IM_GAMUT_SOLARIZE,  ' solarize  = a < level ?  a:  (level * (max-min) - a * (level-min)) / (max-level) param[0]=level percentage (0-100) relative to min-max photography solarization effect.
  IM_GAMUT_SLICE,     ' slice     = start < a || a > end ?  min:  binarize?  max: a param[0]=start,  param[1]=end,  param[2]=binarize IM_GAMUT_EXPAND expand    = a < start ?  min: a > end ? max :  (a-start)*(max-min)/(end-start) + min param[0]=start,  param[1]=end
  IM_GAMUT_CROP,      ' crop      = a < start ?  start: a > end ? end : a param[0]=start,  param[1]=end 
  IM_GAMUT_BRIGHTCONT ' brightcont = a < min ?  min:  a > max ?  max:  a * tan(c_a) + b_s + (max-min)*(1 - tan(c_a))/2 param[0]=bright_shift (-100%..+100%),  param[1]=contrast_factor (-100%..+100%) change brightness and contrast simultaneously.
end enum

' Apply a gamut operation with arguments.
' Supports all data types except IM_CFLOAT.
' The linear operation do a special convertion when min > 0 and max < 1, it forces min=0 and max=1.
' IM_BYTE images have min=0 and max=255 always. \n
' Can be done in place. When there is no extra params, can use NULL.
_DS_ imProcessToneGamut(src_image as const imImage ptr, dst_image as imImage ptr , op as imToneGamut, param as single ptr)

' Converts from (0-1) to (0-255), crop out of bounds values.
' Source image must be IM_FLOAT, and destiny image must be IM_BYTE.
_DS_ imProcessUnNormalize(src_image as const imImage ptr, dst_image as imImage ptr)

' Directly converts IM_USHORT, IM_INT and IM_FLOAT into IM_BYTE images.
' This can also be done using \ref imConvertDataType with IM_CAST_DIRECT.
_DS_ imProcessDirectConv(src_image as const imImage ptr, dst_image as imImage)

' A negative effect. Uses \ref imProcessToneGamut with IM_GAMUT_INVERT for non MAP images.
' Supports all color spaces and all data types except IM_CFLOAT.
' Can be done in place. 
_DS_ imProcessNegative(src_image as const imImage ptr, dst_image as imImage ptr)

' Threshold Operations
' Operations that converts a usually IM_GRAY/IM_BYTE image into a IM_BINARY image using several threshold techniques.

' Apply a manual threshold.
' threshold = a <= level ? 0: value \n
' Normal value is 1 but another common value is 255. Can be done in place for IM_BYTE source.
' Supports all integer IM_GRAY images as source, and IM_BINARY as destiny.
_DS_ imProcessThreshold(src_image as const imImage ptr, dst_image as imImage ptr, level as integer, value as integer)

' Apply a threshold by the difference of two images.
' threshold = a1 <= a2 ? 0: 1
' Can be done in place. 
_DS_ imProcessThresholdByDiff(src_image1 as const imImage ptr, src_image2 as const imImage ptr, dst_image as imImage ptr)

' Apply a threshold by the Hysteresis method.
' Hysteresis thersholding of edge pixels. Starting at pixels with a
' value greater than the HIGH threshold, trace a connected sequence
' of pixels that have a value greater than the LOW threhsold.
' Supports only IM_BYTE images.
_DS_ imProcessHysteresisThreshold(src_image as const imImage ptr, dst_image as imImage ptr, low_thres as integer, high_thres as integer)

' Estimates hysteresis low and high threshold levels.
' Supports only IM_BYTE images.
' Usefull for imProcessHysteresisThreshold.
_DS_ imProcessHysteresisThresEstimate(img as const imImage ptr, low_level as integer ptr, high_level as integer ptr)

' Calculates the threshold level for manual threshold using an uniform error approach.
' Supports only IM_BYTE images.
' Extracted from XITE, Copyright 1991, Blab, UiO
' Returns the used level.
_DF_ imProcessUniformErrThreshold(src_image as const imImage ptr, dst_image as imImage ptr) as integer

' Apply a dithering on each image channel by using a difusion error method.
' It can be applied on any IM_BYTE images. It will "threshold" each channel indivudually, so
' source and destiny must be of the same depth.
_DS_ imProcessDifusionErrThreshold(src_image as const imImage ptr, dst_image as imImage ptr, level as integer)

' Calculates the threshold level for manual threshold using a percentage of pixels
' that should stay bellow the threshold.
' Supports only IM_BYTE images.
_DF_ imProcessPercentThreshold(src_image as const imImage ptr, dst_image as imImage ptr, percent as single) as integer

' Calculates the threshold level for manual threshold using the Otsu approach.
' Returns the used level.
' Supports only IM_BYTE images.
_DF_ imProcessOtsuThreshold(src_image as const imImage ptr, dst_image as imImage ptr) as integer

' Calculates the threshold level for manual threshold using (max-min)/2.
' Returns the used level.
' Supports all integer IM_GRAY images as source, and IM_BINARY as destiny.
_DF_ imProcessMinMaxThreshold(src_image as const imImage ptr, dst_image as imImage ptr) as integer

' Estimates Local Max threshold level for IM_BYTE images.
_DS_ imProcessLocalMaxThresEstimate(img as const imImage ptr, level as integer ptr)

' Apply a manual threshold using an interval.
' threshold = start_level <= a <= end_level ? 1: 0 
' Normal value is 1 but another common value is 255. Can be done in place for IM_BYTE source.
' Supports all integer IM_GRAY images as source, and IM_BINARY as destiny.
_DS_ imProcessSliceThreshold(src_image as const imImage ptr, dst_image as imImage ptr, start_level as integer, end_level as integer)


' Special Effects

' Generates a zoom in effect averaging colors inside a square region.
' Operates only on IM_BYTE images.
_DS_ imProcessPixelate(src_image as const imImage ptr, dst_image as imImage ptr, box_size as integer)

' A simple Posterize effect. It reduces the number of colors in the image eliminating 
' less significant bit planes. Can have 1 to 7 levels. See \ref imProcessBitMask. \n
' Images must have data type IM_BYTE.
_DS_ imProcessPosterize(src_image as const imImage ptr, dst_image as imImage ptr, level as integer)


end extern


#endif ' __IM_PROCESS_PON_BI__

