#ifndef __IM_UTIL_BI__
#define __IM_UTIL_BI__

' see copyright notice at the file iup.txt

#include once "im.bi"

extern "c"

' Utilities
#define IM_MIN(_a, _b) iif(_a < _b, _a, _b)
#define IM_MAX(_a, _b) iif(_a > _b, _a, _b)

' String Utilities

' Check if the two strings are equal.
_DF_ imStrEqual(str1 as const zstring ptr, str2 as const zstring ptr) as integer

' Calculate the size of the string but limited to max_len.
_DF_ imStrNLen(str_ as const zstring ptr, max_len as integer) as integer

' Check if the data is a string.
_DF_ imStrCheck(data_ as const any ptr, count as integer) as integer

' Raw Data Utilities

' Returns the size of the data buffer.
_DF_ imImageDataSize(w as integer, h as integer, color_mode as integer, data_type as integer) as integer

' Returns the size of one line of the data buffer.
' This depends if the components are packed. If packed includes all components, if not includes only one.
_DF_ imImageLineSize(w as integer, color_mode as integer, data_type as integer) as integer

' Returns the number of elements of one line of the data buffer.
' This depends if the components are packed. If packed includes all components, if not includes only one.
_DF_ imImageLineCount(w as integer, color_mode as integer) as integer

' Check if the combination color_mode+data_type is valid.
_DF_ imImageCheckFormat(color_mode as integer, data_type as integer) as integer

' Color Utilities

' Encode RGB components in a long for palete usage.
' "long" definition is compatible with the CD library definition.
_DF_ imColorEncode(red as ubyte, green as ubyte, blue as ubyte) as long

' Decode RGB components from a long for palete usage.
' "long" definition is compatible with the CD library definition.
_DS_ imColorDecode(red as ubyte ptr, green as ubyte ptr, blue as ubyte ptr, color_ as long)

' Color Mode Utilities

' Returns the color mode name.
_DF_ imColorModeSpaceName(color_mode as integer) as const zstring ptr

' Returns the number of components of the color space including alpha.
_DF_ imColorModeDepth(color_mode as integer) as integer

' Returns the color space of the color mode.
#define imColorModeSpace(_cm) (_cm and &HFF)

' Check if the two color modes match. Only the color space is compared.
#define imColorModeMatch(_cm1, _cm2) (imColorModeSpace(_cm1) = imColorModeSpace(_cm2))

' Check if the color mode has an alpha channel.
#define imColorModeHasAlpha(_cm) (_cm and IM_ALPHA)

' Check if the color mode components are packed in one plane.
#define imColorModeIsPacked(_cm) (_cm and IM_PACKED)

' Check if the color mode orients the image from top down to bottom.
#define imColorModeIsTopDown(_cm) (_cm and IM_TOPDOWN)

' Returns the color space of the equivalent display bitmap image.
' Original packing and alpha are ignored. Returns IM_RGB, IM_GRAY, IM_MAP or IM_BINARY.
_DF_ imColorModeToBitmap(color_mode as integer) as integer

' Check if the color mode and data_type defines a display bitmap image.
_DF_ imColorModeIsBitmap(color_mode as integer, data_type as integer) as integer

' Data Type Utilities
type imbyte   as ubyte
type imushort as ushort

#define IM_BYTECROP(_v)      iif(_v < 0, 0, iif(_v >  255,  255, _v))
#define IM_CROPMAX(_v, _max) iif(_v < 0, 0, iif(_v > _max, _max, _v))

' Returns the size in bytes of a specified numeric data type.
_DF_ imDataTypeSize(data_type as integer) as integer

' Returns the numeric data type name given its identifier.
_DF_ imDataTypeName(data_type as integer) as const zstring ptr

' Returns the maximum value of an integer data type. For floating point returns 0.
_DF_ imDataTypeIntMax(data_type as integer) as ulong

' Returns the minimum value of an integer data type. For floating point returns 0. 
_DF_ imDataTypeIntMin(data_type as integer) as long

' Binary Data Utilities

' CPU Byte Orders. 
enum imByteOrder
  IM_LITTLEENDIAN, ' Little Endian - The most significant byte is on the right end of a word. Used by Intel processors.
  IM_BIGENDIAN     ' Big Endian - The most significant byte is on the left end of a word. Used by Motorola processors, also is the network standard byte order.
end enum

' Returns the current CPU byte order.
_DF_ imBinCPUByteOrder() as integer

' Changes the byte order of an array of 2, 4 or 8 byte values.
_DS_ imBinSwapBytes(data_ as any ptr, count as integer, size as integer)

' Changes the byte order of an array of 2 byte values.
_DS_ imBinSwapBytes2(data_ as any ptr, count as integer)

' Inverts the byte order of the 4 byte values 
_DS_ imBinSwapBytes4(data_ as any ptr, count as integer)
            
' Inverts the byte order of the 8 byte values 
_DS_ imBinSwapBytes8(data_ as any ptr, count as integer)

' Data Compression Utilities

' Compresses the data using the ZLIB Deflate compression.
' The destination buffer must be at least 0.1% larger than source_size plus 12 bytes.
' It compresses raw byte data. zip_quality can be 1 to 9.
' Returns the size of the compressed buffer or zero if failed.
_DF_ imCompressDataZ(src_data as const any ptr, src_size as integer, dst_data as any ptr, dst_size as integer, zip_quality as integer) as integer

' Uncompresses the data compressed with the ZLIB Deflate compression.
' Returns zero if failed.
_DF_ imCompressDataUnZ(src_data as const any ptr, src_size as integer, dst_data as any ptr, dst_size as integer) as integer

' Compresses the data using the libLZF compression.
' Returns the size of the compressed buffer or zero if failed.
_DF_ imCompressDataLZF(src_data as const any ptr, src_size as integer, dst_data as any ptr, dst_size as integer, zip_quality as integer) as integer

' Uncompresses the data compressed with the libLZF compression.
' Returns zero if failed.
_DF_ imCompressDataUnLZF(src_data as const any ptr, src_size as integer, dst_data as any ptr, dst_size as integer) as integer

end extern

#endif ' __IM_UTIL_BI__
