#ifndef __IUP_ATTRIB_OPT_BI__ 
#define __IUP_ATTRIB_OPT_BI__

' see copyright notice at the file iup.txt

' optional

#include once "iup.bi"

extern "C"


' Attribute Environment 
' When attributes are not stored at the control they are stored in a hash table. (see iup_table.bi)
' As a general rule use:
' IupGetAttribute, IupSetAttribute, ... : when care about control implementation, hash table, inheritance and default value
' - iupAttribGetStr,Int,Float: when care about inheritance, hash table and default value
' - iupAttribGet,... : ONLY access the hash table
' These different functions have very different performances and results. So use them wiselly.

' Returns true if the attribute name if in the internal format "_IUP...".
#define iupATTRIB_ISINTERNAL(_name) iif(_name[0] = asc("_") and _name[1] = asc("I") and _name[2] = asc("U")  and _name[3] = asc("P"), 1, 0)

' Returns true if the attribute name is a known pointer.
_DF_ iupAttribIsPointer(ih as IHandle ptr, name_ as const zstring ptr) as integer

' Sets the attribute only in the hash table as a pointer.
' It ignores children.
_DS_ iupAttribSetStr(ih as IHandle ptr, name_ as const zstring ptr, value as const zstring ptr)

' Sets the attribute only in the hash table as a string. 
' The string is internally duplicated.
' It ignores children.
_DS_ iupAttribStoreStr(ih as IHandle ptr, name_ as const zstring ptr, value as const zstring ptr)

' Sets the attribute only in the hash table as a string. 
' The string is internally duplicated. Use same format as sprintf.
' It ignores children.
_DS_ iupAttribSetStrf(ih as IHandle ptr, name_ as const zstring ptr, format_ as const zstring ptr, ...)

' Sets an integer attribute only in the hash table.
' It will be stored as a string.
' It ignores children.
_DS_ iupAttribSetInt(ih as IHandle ptr, name_ as const zstring ptr, num as integer) 

' Sets an floating point attribute only in the hash table.
' It will be stored as a string.
' It ignores children.
_DS_ iupAttribSetFloat(ih as IHandle ptr, name_ as const zstring ptr, num as single)

' Returns the attribute from the hash table only. 
' NO inheritance, NO control implementation, NO defalt value here.
_DF_ iupAttribGet(ih as IHandle ptr, name_ as const zstring ptr) as zstring ptr

' Returns the attribute from the hash table only, 
' but if not defined then checks in its parent tree.
' NO control implementation, NO defalt value here.
_DF_ iupAttribGetInherit(ih as IHandle ptr, name_ as const zstring ptr) as zstring ptr

' Returns the attribute from the hash table of a native parent.
' Don't check for default values. Don't check at the element.
' Used for BGCOLOR and BACKGROUND attributes.
_DF_ iupAttribGetInheritNativeParent(ih as IHandle ptr, name_ as const zstring ptr) as zstring ptr

' Returns the attribute from the hash table as a string, 
' but if not defined then checks in its parent tree if allowed by the control implementation, 
' if still not defined then returns the registered default value if any.
' NO control implementation, only checks inheritance and default value from it.
_DF_ iupAttribGetStr(ih as IHandle ptr, name_ as const zstring ptr) as zstring ptr   

' Returns the attribute from the hash table as a string, 
' but if not defined then checks in the control implementation, 
' if still not defined then returns the registered default value if any. \n
' NO inheritance here. Used only in the IupLayoutDialog.
_DF_ iupAttribGetLocal(ih as IHandle ptr, name_ as const zstring ptr) as zstring ptr

' Same as iupAttribGetStr but returns an integer number.
' Checks also for boolean values.
_DF_ iupAttribGetInt(ih as IHandle ptr, name_ as const zstring ptr) as integer

' Same as \ref iupAttribGetStr but checks for boolean values.
' Use iupStrBoolean.
 _DF_ iupAttribGetBoolean(ih as IHandle ptr, name_ as const zstring ptr) as integer

' Same as \ref iupAttribGetStr but returns an floating point number.
_DF_ iupAttribGetFloat(ih as IHandle ptr, name_ as const zstring ptr) as single

' Set an internal name to a handle.
_DS_ iupAttribSetHandleName(ih as IHandle ptr)

' Returns the internal name if set.
_DF_ iupAttribGetHandleName(ih as IHandle ptr) as zstring ptr

' For all attributes in the evironment, call the class SetAttribute only.
' Called only after the element is mapped, but before the children are mapped.
_DS_ iupAttribUpdate(ih as IHandle ptr) 

' For all registered inherited attributes, checks the parent tree and 
' call the class SetAttribute if the attribute is defined.
' Called only after the element is mapped, but before the children are mapped.
_DS_ iupAttribUpdateFromParent(ih as IHandle ptr)

' For all attributes in the evironment, call the class SetAttribute only for the children.
' Called only after the element is mapped, and after the children are mapped.
_DS_ iupAttribUpdateChildren(ih as IHandle ptr)

end extern


#endif ' __IUP_ATTRIB_OPT_BI__
