#ifndef __IUP_CLASSBASE_OPT_BI__
#define __IUP_CLASSBASE_OPT_BI__

' see copyright notice at the file iup.txt

' optional 

extern "C"

' Base Class

' Register all common base attributes:
' WID, SIZE, RASTERSIZE, POSITION, FONT (and derived)
' All controls that are positioned inside a dialog must register all common base attributes.
_DS_ iupBaseRegisterCommonAttrib(ic as Iclass ptr)

' Register all visual base attributes:
' VISIBLE, ACTIVE, ZORDER, X, Y , TIP (and derived)
' All controls that are positioned inside a dialog must register all visual base attributes.
_DS_ iupBaseRegisterVisualAttrib(ic as Iclass ptr)

' Register all common callbacks:
' MAP_CB, UNMAP_CB, GETFOCUS_CB, KILLFOCUS_CB, ENTERWINDOW_CB, LEAVEWINDOW_CB, K_ANY, HELP_CB.
'_DS_ iupBaseRegisterCommonCallbacks(ic as Iclass ptr)

' Register driver dependent common attributes. 
' Used only from iupBaseRegisterCommonAttrib
'_DS_ iupdrvBaseRegisterCommonAttrib(ic as Iclass ptr)

' Updates the expand member of the IUP object from the EXPAND attribute.
' Should be called in the beginning of the ComputeNaturalSize for a container.
_DS_ iupBaseContainerUpdateExpand(ih as IHandle ptr)

' Initializes the natural size using the user size, then
' if a container then update the "expand" member from the EXPAND attribute, then
' call iupClassObjectComputeNaturalSize for containers if they have children or
' call iupClassObjectComputeNaturalSize for non-containers if user size is not defined.
' Must be called for each children in the container.
' First call is in iupLayoutCompute.
_DS_ iupBaseComputeNaturalSize(ih as IHandle ptr)

' Update the current size from the available size, the natural size, expand and shrink.
' Call iupClassObjectSetChildrenCurrentSize for containers if they have children.
' Must be called for each children in the container.
' First call is in iupLayoutCompute.
_DS_ iupBaseSetCurrentSize(ih as IHandle ptr, w as integer, h as integer, shrink as integer)

' Set the current position and update children position for containers.
' Call iupClassObjectSetChildrenPosition for containers if they have children.
' Must be called for each children in the container.
' First call is in iupLayoutCompute.
_DS_ iupBaseSetPosition(ih as IHandle ptr, x as integer, y as integer)

' Updates the SIZE attribute if defined. 
' Called only from iupdrvSetStandardFontAttrib.
'_DS_ iupBaseUpdateSizeFromFont(ih as IHandle ptr)

' Base Class Methods

' Driver dependent Iclass::LayoutUpdate method.
'_DS_ iupdrvBaseLayoutUpdateMethod(ih as IHandle ptr)

' Driver dependent Iclass::UnMap method.
'_DS_ iupdrvBaseUnMapMethod(ih as IHandle ptr)

' Native type void Iclass::Map method.
_DF_ iupBaseTypeVoidMapMethod(ih as IHandle ptr) as integer


' Base Class Attribute Functions
' Used by the controls for iupClassRegisterAttribute. 

' common
_DF_ iupBaseGetWidAttrib         (ih as IHandle ptr) as zstring ptr
'_DF_ iupBaseSetNameAttrib        (ih as IHandle ptr, value as const zstring ptr) as integer
_DF_ iupBaseSetRasterSizeAttrib  (ih as IHandle ptr, value as const zstring ptr) as integer
_DF_ iupBaseSetSizeAttrib        (ih as IHandle ptr, value as const zstring ptr) as integer
_DF_ iupBaseGetSizeAttrib        (ih as IHandle ptr) as zstring ptr
_DF_ iupBaseGetRasterSizeAttrib  (ih as IHandle ptr) as zstring ptr
'_DF_ iupBaseGetClientOffsetAttrib(ih as IHandle ptr) as zstring ptr

' visual
_DF_ iupBaseGetVisibleAttrib      (ih as IHandle ptr) as zstring ptr
_DF_ iupBaseSetVisibleAttrib      (ih as IHandle ptr, value as const zstring ptr) as integer
_DF_ iupBaseGetActiveAttrib       (ih as IHandle ptr) as zstring ptr
_DF_ iupBaseSetActiveAttrib       (ih as IHandle ptr, value as const zstring ptr) as integer
'_DF_ iupdrvBaseSetZorderAttrib    (ih as IHandle ptr, value as const zstring ptr) as integer
'_DF_ iupdrvBaseSetTipAttrib       (ih as IHandle ptr, value as const zstring ptr) as integer
'_DF_ iupdrvBaseSetTipVisibleAttrib(ih as IHandle ptr, value as const zstring ptr) as integer
'_DF_ iupdrvBaseGetTipVisibleAttrib(ih as IHandle ptr) as zstring ptr
'_DF_ iupdrvBaseSetBgColorAttrib   (ih as IHandle ptr, value as const zstring ptr) as integer
'_DF_ iupdrvBaseSetFgColorAttrib   (ih as IHandle ptr, value as const zstring ptr) as integer
'_DF_ iupBaseNativeParentGetBgColorAttrib(ih as IHandle ptr) as zstring ptr

' other
_DF_ iupBaseContainerGetExpandAttrib(ih as IHandle ptr) as zstring ptr
'_DF_ iupdrvBaseSetCursorAttrib(ih as IHandle ptr, value as const zstring ptr) as integer
'_DF_ iupBaseNoSaveCheck(ih as IHandle ptr, name_ as const zstring ptr) as integer

' Windows Only
'_DF_ iupdrvBaseGetTitleAttrib(ih as IHandle ptr) as zstring ptr
'_DF_ iupdrvBaseSetTitleAttrib(ih as IHandle ptr, value as const zstring ptr) as integer

' Base Class Utilities
#define iupMAX(_a,_b) iif(_a>_b,_a,_b)
#define iupROUND(_x) cint(iff(_x>0,_x+0.5,_x-0.5))
#define iupCOLOR8TO16(_x) cushort(_x*257)  ' 65535/255 = 257
#define iupCOLOR16TO8(_x) cubyte(_x/257)
#define iupBYTECROP(_x) cubyte(iif(_x<0,0,iif(_x>255,255,_x)))
enum 
  IUP_ALIGN_ALEFT 
  IUP_ALIGN_ACENTER 
  IUP_ALIGN_ARIGHT
end enum

#define IUP_ALIGN_ABOTTOM IUP_ALIGN_ARIGHT
#define IUP_ALIGN_ATOP IUP_ALIGN_ALEFT

enum 
  IUP_SB_NONE
  IUP_SB_HORIZ
  IUP_SB_VERT
end enum
_DF_ iupBaseGetScrollbar(ih as IHandle ptr) as integer

'_DF_ iupBaseNativeParentGetBgColor(ih as IHandle ptr) any zstring ptr 
_DS_ iupBaseCallValueChangedCb(ih as IHandle ptr)

end extern


#endif ' __IUP_CLASSBASE_BI__
