#ifndef __IUP_OBJECT_OPT_BI__
#define __IUP_OBJECT_OPT_BI__

' see copyright notice at the file iup.txt

' optional

#include once "crt/stdarg.bi"
#include once "iup_class_opt.bi"

extern "c"

' Ihandle Object
' Object handle for all the elements.

' SIZE to RASTERSIZE
#define iupWIDTH2RASTER(_w, _cw)  int ( ((_w * _cw)/4.0 + 0.5) )
' SIZE to RASTERSIZE
#define iupHEIGHT2RASTER(_h, _ch) int ( ((_h * _ch)/8.0 + 0.5) )
' RASTERSIZE to SIZE
#define iupRASTER2WIDTH(_w, _cw)  int ( ((_w * 4.0)/_cw + 0.5) )
' RASTERSIZE to SIZE
#define iupRASTER2HEIGHT(_h, _ch) int ( ((_h * 8.0)/_ch + 0.5) )


' Expand configuration
enum Iexpand
  IUP_EXPAND_NONE = &H00,
  IUP_EXPAND_H0   = &H01,  ' only set by IupFill
  IUP_EXPAND_H1   = &H02,
  IUP_EXPAND_W0   = &H04,  ' only set by IupFill
  IUP_EXPAND_W1   = &H08
end enum

' Expand configuration
#define IUP_EXPAND_WIDTH (IUP_EXPAND_W1 or IUP_EXPAND_W0)
' Expand configuration
#define IUP_EXPAND_HEIGHT (IUP_EXPAND_H1 or IUP_EXPAND_H0)
' Expand configuration
#define IUP_EXPAND_BOTH (IUP_EXPAND_WIDTH or IUP_EXPAND_HEIGHT)


' A simple definition that do not depends on the native system, 
' but helps a lot when writing native code.
#if defined(GTK_MAJOR_VERSION)
type InativeHandle as _GtkWidget
#elseif defined(XmVERSION)
type InativeHandle as _WidgetRec
#elseif defined(WINVER)
type InativeHandle as HWND__
#else
type _InativeHandle as any : type InativeHandle as _InativeHandle ptr
#endif

' Each control may define its own structure in its private module.
type _IcontrolData  as any : type IcontrolData as _IcontrolData

' IcontrolData allocation utility.
#define iupALLOCCTRLDATA() cptr(as IcontrolData ptr,calloc(1, sizeof(IcontrolData)))

' General flags.
enum Iflags
  IUP_FLOATING         = &H01, ' is a floating element. FLOATING=Yes
  IUP_FLOATING_IGNORE  = &H02, ' is a floating element. FLOATING=Ignore. Do not compute layout.
  IUP_MAXSIZE          = &H04, ' has the MAXSIZE attribute set
  IUP_MINSIZE          = &H08, ' has the MAXSIZE attribute set
  IUP_INTERNAL         = &H10  ' it is an internal element of the container
end enum

' Structure used by all the elements.
type Ihandle_
  as zstring * 4 sig      ' IUP Signature, initialized with "IUP", cleared on destroy
  as Iclass_ ptr iclass   ' Ihandle Class
  as Itable ptr attrib    ' attributes table
  as integer serial       ' serial number used for controls that need a numeric id, initialized with -1
  as InativeHandle handle ' native handle. initialized when mapped. InativeHandle definition is system dependent.
  as integer expand       ' expand configuration, a combination of Iexpand, for containers is a combination of the children expand's
  as Iflags flags         ' flags configuration, a combination of Iflags
  as integer x, y         ' upper-left corner relative to the native parent. always 0 for the dialog.
  as integer userwidth    ' user defined size for the control using SIZE or RASTERSIZE   
  as integer userheight 
  as integer naturalwidth ' the calculated size based in the control contents and the user size
  as integer naturalheight 
  as integer currentwidth ' actual size of the control in pixels (window size, including decorations and margins).
  as integer currentheight
  as Ihandle_ ptr parent     ' previous control in the hierarchy tree
  as Ihandle_ ptr firstchild ' first child control in the hierarchy tree
  as Ihandle_ ptr brother    ' next control inside parent
  as IcontrolData ptr data   ' private control data. automatically freed if not NULL in destroy
end type


' Creates an object initializes iclass and nativetype.
' Called only from IupCreate and IupLoad. */
'_DF_ iupObjectCreate (ic as Iclass ptr, params as any ptr ptr) as Ihandle ptr


' Utility that returns an array of parameters. Must call free for the returned value after usage.
' Used by the creation functions of objects that receives a NULL terminated array of parameters.
_DF_ iupObjectGetParamList(first as any ptr, arglist as va_list) as any ptr ptr
 
' Checks if the handle is still valid based on the signature.
' But if the handle was destroyed still can access invalid memory.
_DF_ iupObjectCheck(ih as Ihandle ptr) as integer

end extern

/' Other functions declared in <iup.h> and implemented here. 
IupCreate 
IupCreatev
IupCreatep
IupDestroy
'/



#endif '__IUP_OBJECT_OPT_BI__

