#ifndef __OPENB3D_BI__
#define __OPENB3D_BI__

#include once "GL/gl.bi"
#include once "GL/glu.bi"

#ifdef __FB_WIN32__
 #ifndef IUP_DLL
  '#inclib "gcc"
  #inclib "gcc_eh"
  #inclib "stdc++"
  #inclib "gLee"
  #libpath "lib/win32/"
 #else
  #libpath "lib/win32dll/"
 #endif
#else
 #libpath "lib/linux/"
#endif
#inclib "openb3d"


' collisions
enum COLLISION_METHODS
  COLLISION_METHOD_ELLIPSOID = 1
  COLLISION_METHOD_POLYGONE
  COLLISION_METHOD_BOX
end enum

enum COLLISION_RESPONCES
  COLLISION_RESPONCE_NO = 0
  COLLISION_RESPONCE_STOP
  COLLISION_RESPONCE_SLIDE1 ' full sliding collision
  COLLISION_RESPONCE_SLIDE2 ' prevent entities from sliding down slopes
end enum

' brushes
enum BRUSH_FLAGS
  BRUSH_FLAG_COLOR      =  1 ' (default)
  BRUSH_FLAG_ALPHA      =  2
  BRUSH_FLAG_MASKED     =  4
  BRUSH_FLAG_MIPMAPPED  =  8 
  BRUSH_FLAG_CLAMPU     = 16 
  BRUSH_FLAG_CLAMPV     = 32
  BRUSH_FLAG_SPHERICAL  = 64
end enum

enum BRUSH_BLEND_MODES
  BRUSH_BLEND_MODE_ALPHA = 1 ' (default)
  BRUSH_BLEND_MODE_MULTIPLY
  BRUSH_BLEND_MODE_ADD
end enum

enum BRUSH_FX_MODES
  BRUSH_FX_MODE_NOTHING           =  0 '(default)
  BRUSH_FX_MODE_FULLBRIGHT        =  1
  BRUSH_FX_MODE_VERTEXCOLOR       =  2
  BRUSH_FX_MODE_FLATSHADED        =  4
  BRUSH_FX_MODE_NOFOG             =  8
  BRUSH_FX_MODE_NOBACKFACECULLING = 16
  BRUSH_FX_MODE_ALPHABLENDING     = 32 ' only for Entity
end enum

' lights
enum LIGHT_TYPES
  LIGHT_TYPE_DIRECTIONAL = 1 '(default)
  LIGHT_TYPE_POINT
  LIGHT_TYPE_SPOT
end enum  

' sprites
enum SPRITE_VIEW_MODES
  SPRITE_VIEW_MODE_FIXED = 1 ' (sprite always faces camera - default)
  SPRITE_VIEW_MODE_FREE      ' (sprite is independent of camera)
  SPRITE_VIEW_MODE_UPRIGHT1  ' (sprite always faces camera, but rolls with camera as well, unlike mode no.1)
  SPRITE_VIEW_MODE_UPRIGHT2  ' (sprite always remains upright. Gives a 'billboard' effect. Good for trees, spectators etc.)
end enum

' textures
enum TEXTURE_BLEND_MODES
  TEXTURE_BLEND_MODE_NO = 0
  TEXTURE_BLEND_MODE_NO_ALPHA  ' (alpha when texture loaded with alpha flag)
  TEXTURE_BLEND_MODE_MULTIPLY  ' (default)
  TEXTURE_BLEND_MODE_ADD
  TEXTURE_BLEND_MODE_DOT3 
  TEXTURE_BLEND_MODE_MULTIPLY2
end enum

enum TEXTURE_COORD_SETS
  TEXTURE_COORD_SET_FIRST = 0 ' (default)
  TEXTURE_COORD_SET_SECOND
end enum

enum TEXTURE_CUBE_FACES
  TEXTURE_CUBE_FACE_LEFT = 0 ' (negative X) face
  TEXTURE_CUBE_FACE_FORWARD  ' (positive Z) face - (default)
  TEXTURE_CUBE_FACE_RIGHT    ' (positive X) face
  TEXTURE_CUBE_FACE_BACKWARD ' (negative Z) face
  TEXTURE_CUBE_FACE_UP       ' (positive Y) face
  TEXTURE_CUBE_FACE_DOWN     ' (negative Y) face
end enum

enum TEXTURE_CUBE_MODES
  TEXTURE_CUBE_MODE_SPECULAR = 1 '(default)
  TEXTURE_CUBE_MODE_DIFFUSE
  TEXTURE_CUBE_MODE_REFRATION
end enum  

enum TEXTURE_FLAGS
  TEXTURE_FLAG_COLOR           =   1 '(default)
  TEXTURE_FLAG_ALPHA           =   2
  TEXTURE_FLAG_MASKED          =   4
  TEXTURE_FLAG_MIPMAPPED       =   8
  TEXTURE_FLAG_CLAMPU          =  16
  TEXTURE_FLAG_CLAMPV          =  32
  TEXTURE_FLAG_SPHERICAL       =  64
  TEXTURE_FLAG_CUBIC           = 128 
  TEXTURE_FLAG_STORE_VRAM      = 256
  TEXTURE_FLAG_FORCE_HIGHCOLOR = 512
end enum

' entities
enum ENTITY_PICK_MODES
  ENTITY_PICK_MODE_UNPICKABLE = 0 ' (default)
  ENTITY_PICK_MODE_SPHERE         ' (EntityRadius is used)
  ENTITY_PICK_MODE_POLYGONE
  ENTITY_PICK_MODE_BOX            ' (EntityBox is used)
end enum

enum ENTITY_ANIM_MODES
  ENTITY_ANIM_MODE_STOP = 0 
  ENTITY_ANIM_MODE_LOOP      ' (default)
  ENTITY_ANIM_MODE_PING_PONG
  ENTITY_ANIM_MODE_ONE_SHOT
end enum

' cameras
enum  CAMERA_PROJECTION_MODES
  CAMERA_PROJECTION_MODE_NO = 0      ' disables camera (faster than HideEntity)
  CAMERA_PROJECTION_MODE_PERSPECTIVE ' (default)
  CAMERA_PROJECTION_MODE_OTHOGRAPHIC
end enum

enum  CAMERA_FOG_MODES
  CAMERA_FOG_MODE_NO = 0 '(default)
  CAMERA_FOG_MODE_LINEARE
end enum

' meshes
enum CSG_METHODS
  CSG_METHOD_SUBTRACTION = 0
  CSG_METHOD_UNION          ' (default)
  CSG_METHOD_INTERSECTION
end enum

' stencil
enum STENCIL_MODES
  STENCIL_MODE_CULL_INCR = -2
  STENCIL_MODE_DECR = -1
  STENCIL_MODE_INCR = 1
  STENCIL_MODE_CULL_DECR = 2
end enum  

enum STENCIL_OPERATORS
  STENCIL_OPERATOR_NOT_EQUAL = 0
  STENCIL_OPERATOR_EQUAL
  STENCIL_OPERATOR_SHORTER_EQUAL
  STENCIL_OPERATOR_GREATER_EQUAL
end enum


type B3D_ENTITY       as any
type B3D_CAMERA       as any
type B3D_LIGHT        as any
type B3D_TERRAIN      as any
type B3D_BRUSH        as any
type B3D_MATERIAL     as any
type B3D_SHADER       as any
type B3D_TEXTURE      as any
type B3D_MESH         as any
type B3D_SURFACE      as any
type B3D_SPRITE       as any 
type B3D_VOXELSPRITE  as any 
type B3D_STENCIL      as any
type B3D_SHADOWOBJECT as any

extern "c"

' GLOBAL
declare sub Graphics3D  (w as integer, h as integer, depth as integer=0, mode as integer=0, rate as integer=60)
declare sub AntiAlias   (samples as integer)
declare sub Wireframe   (enable as integer)

declare sub AmbientLight(r as single, g as single, b as single)
declare sub ClearTextureFilters () 

declare sub RenderWorld()
declare sub UpdateWorld (anim_speed as single=1)
declare sub ClearWorld (entities as integer=1, brushes as integer=1, textures as integer=1)

declare sub Collisions(src_no as integer, dest_no as integer, method as COLLISION_METHODS, response as COLLISION_RESPONCES=0)

' tools (texture)
declare sub BackBufferToTex(tex as B3D_TEXTURE ptr, frame as integer=0)
declare sub BufferToTex    (tex as B3D_TEXTURE ptr, buffer as any ptr, frame as integer)
declare sub TexToBuffer    (tex as B3D_TEXTURE ptr, buffer as any ptr, frame as integer)
declare sub CameraToTex    (tex as B3D_TEXTURE ptr, cam as any ptr, frame as integer=0)

' math
declare function VectorPitch(vx as single, vy as single, vz as single) as single
declare function VectorYaw  (vx as single, vy as single, vz as single) as single

' BRUSH
declare function LoadBrush  (file as zstring ptr, flags as BRUSH_FLAGS = BRUSH_FLAG_COLOR, u_scale as single=1, v_scale as single=1) as B3D_BRUSH ptr
declare function CreateBrush(r as single=255, g as single=255, b as single=255) as B3D_BRUSH ptr
declare sub FreeBrush  (brush as B3D_BRUSH ptr)

declare sub BrushColor     (brush as B3D_BRUSH ptr, r as single, g as single, b as single)
declare sub BrushAlpha     (brush as B3D_BRUSH ptr, a as single)
declare sub BrushShininess (brush as B3D_BRUSH ptr, s as single)
declare sub BrushBlend     (brush as B3D_BRUSH ptr, BlendMode as BRUSH_BLEND_MODES = BRUSH_BLEND_MODE_ALPHA)
declare sub BrushFX        (brush as B3D_BRUSH ptr, FXMode as BRUSH_FX_MODES = BRUSH_FX_MODE_NOTHING)
declare sub BrushTexture   (brush as B3D_BRUSH ptr, tex as B3D_TEXTURE ptr, frame as integer=0, index as integer=0)
declare function GetBrushTexture(brush as B3D_BRUSH ptr, index as integer=0) as B3D_TEXTURE ptr

' CAMERA
declare function CreateCamera(parent as B3D_ENTITY ptr=0) as B3D_CAMERA ptr
declare sub CameraClsColor (cam as B3D_CAMERA ptr, r as single, g as single, b as single)
declare sub CameraClsMode  (cam as B3D_CAMERA ptr, cls_depth as integer, cls_zbuffer as integer)
declare sub CameraFogColor (cam as B3D_CAMERA ptr, r as single, g as single, b as single)
declare sub CameraFogMode  (cam as B3D_CAMERA ptr, mode as CAMERA_FOG_MODES)
declare sub CameraFogRange (cam as B3D_CAMERA ptr, nnear as single, nfar as single)
declare function CameraPick(cam as B3D_CAMERA ptr, x as single, y as single) as B3D_ENTITY ptr
declare sub CameraProject  (cam as B3D_CAMERA ptr, x as single, y as single, z as single)
declare sub CameraProjMode (cam as B3D_CAMERA ptr, mode as CAMERA_PROJECTION_MODES)
declare sub CameraRange    (cam as B3D_CAMERA ptr, nnear as single, nfar as single)
declare sub CameraViewport (cam as B3D_CAMERA ptr, x as integer, y as integer, w as integer, h as integer)
declare sub CameraZoom     (cam as B3D_CAMERA ptr, zoom as single)


' ENTITY
declare sub FreeEntity(ent as B3D_ENTITY ptr)
declare function CopyEntity(ent as B3D_ENTITY ptr, parent as B3D_ENTITY ptr=0) as B3D_ENTITY ptr
declare sub ResetEntity(ent as B3D_ENTITY ptr)
declare function EntityClass(ent as B3D_ENTITY ptr) as zstring ptr
declare sub EntityType(ent as B3D_ENTITY ptr, type_no as integer, recursive as integer=0)
declare function GetEntityType(ent as B3D_ENTITY ptr) as integer

' ENTITY (matrix)
declare function GetMatElement (ent as B3D_ENTITY ptr, row as integer, col as integer) as single

#ifdef PRIVATE_BUILD
' get/set position & rotation (no scaling)
declare sub TransformEntity(ent as B3D_ENTITY ptr, pMatrix4x4 as single ptr)
declare sub EntityTransform(ent as B3D_ENTITY ptr, pMatrix4x4 as single ptr)
#endif

' ENTITY position
declare sub PositionEntity (ent as B3D_ENTITY ptr, x as single, y as single, z as single, glob as integer=0)
declare sub MoveEntity     (ent as B3D_ENTITY ptr, x as single, y as single, z as single)
declare sub TranslateEntity(ent as B3D_ENTITY ptr, x as single, y as single, z as single, glob as integer=0)
declare function EntityX(ent as B3D_ENTITY ptr, glob as integer=0) as single
declare function EntityY(ent as B3D_ENTITY ptr, glob as integer=0) as single
declare function EntityZ(ent as B3D_ENTITY ptr, glob as integer=0) as single

' ENTITY rotation
declare sub      RotateEntity(ent as B3D_ENTITY ptr, x as single, y as single, z as single, glob as integer=0)
declare sub      TurnEntity  (ent as B3D_ENTITY ptr, x as single, y as single, z as single, glob as integer=0)
declare sub      PointEntity (ent as B3D_ENTITY ptr, target_ent as B3D_ENTITY ptr, roll as single=0)
declare function EntityPitch (ent as B3D_ENTITY ptr, glob as integer=0) as single
declare function EntityYaw   (ent as B3D_ENTITY ptr, glob as integer=0) as single
declare function EntityRoll  (ent as B3D_ENTITY ptr, glob as integer=0) as single

declare function DeltaPitch  (ent1 as any ptr, ent2 as any ptr) as single
declare function DeltaYaw    (ent1 as any ptr, ent2 as any ptr) as single

' ENTITY size
declare sub      ScaleEntity (ent as B3D_ENTITY ptr, x as single, y as single, z as single, glob as integer=0)
declare function EntityScaleX(ent as B3D_ENTITY ptr, glob as integer=0) as single
declare function EntityScaleY(ent as B3D_ENTITY ptr, glob as integer=0) as single
declare function EntityScaleZ(ent as B3D_ENTITY ptr, glob as integer=0) as single

declare sub MeshCullRadius(ent as B3D_ENTITY ptr, radius as single)

declare sub NameEntity(ent as B3D_ENTITY ptr, name as zstring ptr)
declare function EntityName(ent as B3D_ENTITY ptr) as zstring ptr

declare sub ShowEntity(ent as B3D_ENTITY ptr)
declare sub HideEntity(ent as B3D_ENTITY ptr)
declare function EntityVisible(src_ent as B3D_ENTITY ptr, dest_ent as B3D_ENTITY ptr) as integer

declare sub PaintEntity(ent as B3D_ENTITY ptr, brush as B3D_BRUSH ptr)
declare sub ShadeEntity(ent as B3D_ENTITY ptr, materila as B3D_MATERIAL ptr) 

declare sub EntityAutoFade (ent as B3D_ENTITY ptr, near as single, far as single)

' ENTITY (brush)
declare sub EntityColor    (ent as B3D_ENTITY ptr, r as single, g as single, b as single)
declare sub EntityAlpha    (ent as B3D_ENTITY ptr, a as single)
declare sub EntityShininess(ent as B3D_ENTITY ptr, shine as single)
declare sub EntityTexture  (ent as B3D_ENTITY ptr, tex as B3D_TEXTURE ptr, frame as integer=0, index as integer=0)
declare sub EntityBlend    (ent as B3D_ENTITY ptr, blend as BRUSH_BLEND_MODES)
declare sub EntityFX       (ent as B3D_ENTITY ptr, fx as BRUSH_FX_MODES)

declare function GetEntityBrush (ent as B3D_ENTITY ptr) as B3D_BRUSH ptr

declare function EntityDistance (ent as B3D_ENTITY ptr, ent2 as any ptr) as single
declare function EntityInView   (ent as B3D_ENTITY ptr, cam as any ptr) as integer
declare sub EntityOrder    (ent as B3D_ENTITY ptr, order as integer)

declare function EntityPick    (ent as B3D_ENTITY ptr, range as single) as B3D_ENTITY ptr
declare sub EntityPickMode(ent as B3D_ENTITY ptr, pick_mode as ENTITY_PICK_MODES, obscurer as integer=1)
declare sub EntityRadius  (ent as B3D_ENTITY ptr, radius_x as single, radius_y as single=0)
declare sub EntityBox     (ent as B3D_ENTITY ptr, x as single, y as single, z as single, w as single, h as single, d as single)


' ENTITY (hierarchy)
declare sub EntityParent   (ent as B3D_ENTITY ptr, parent_ent as B3D_ENTITY ptr, glob as integer=1)
declare function GetParentEntity(ent as B3D_ENTITY ptr) as B3D_ENTITY ptr
declare function FindChild      (ent as B3D_ENTITY ptr, child_name as zstring ptr) as B3D_ENTITY ptr
declare function CountChildren  (ent as B3D_ENTITY ptr) as integer
declare function GetChild       (ent as B3D_ENTITY ptr, child_no as integer) as B3D_ENTITY ptr

' ENTITY (animation)
declare sub Animate        (ent as B3D_ENTITY ptr, mode as ENTITY_ANIM_MODES=ENTITY_ANIM_MODE_LOOP, speed as single=1.0, seq as integer=0, trans_ as integer=0)
declare function Animating      (ent as B3D_ENTITY ptr) as integer
declare function AnimLength     (ent as B3D_ENTITY ptr) as integer
declare function AnimSeq        (ent as B3D_ENTITY ptr) as integer
declare function AnimTime       (ent as B3D_ENTITY ptr) as single
declare function AddAnimSeq     (ent as B3D_ENTITY ptr, length as integer) as integer
declare function ExtractAnimSeq (ent as B3D_ENTITY ptr, first_frame as integer, last_frame as integer, seq as integer=0) as integer
declare sub SetAnimTime    (ent as B3D_ENTITY ptr, time_ as single, seq as integer=0)

' ENTITY (collision)
declare function EntityCollided (ent as B3D_ENTITY ptr, type_no as integer) as B3D_ENTITY ptr
declare function CollisionEntity(ent as B3D_ENTITY ptr, index as integer) as B3D_ENTITY ptr
declare function CountCollisions(ent as B3D_ENTITY ptr) as integer

declare function CollisionSurface (ent as B3D_ENTITY ptr, index as integer) as B3D_SURFACE ptr
declare function CollisionTime    (ent as B3D_ENTITY ptr, index as integer) as single
declare function CollisionTriangle(ent as B3D_ENTITY ptr, index as integer) as integer

declare function CollisionNX(ent as B3D_ENTITY ptr, index as integer) as single
declare function CollisionNY(ent as B3D_ENTITY ptr, index as integer) as single
declare function CollisionNZ(ent as B3D_ENTITY ptr, index as integer) as single

declare function CollisionX   (ent as B3D_ENTITY ptr, index as integer) as single
declare function CollisionY   (ent as B3D_ENTITY ptr, index as integer) as single
declare function CollisionZ   (ent as B3D_ENTITY ptr, index as integer) as single

declare sub TFormNormal(x as single, y as single, z as single, src_ent as B3D_ENTITY ptr, dest_ent as B3D_ENTITY ptr)
declare sub TFormPoint (x as single, y as single, z as single, src_ent as B3D_ENTITY ptr, dest_ent as B3D_ENTITY ptr)
declare sub TFormVector(x as single, y as single, z as single, src_ent as B3D_ENTITY ptr, dest_ent as B3D_ENTITY ptr)

declare function TFormedX() as single
declare function TFormedY() as single
declare function TFormedZ() as single


' LIGHT
declare function CreateLight    (lighttype as LIGHT_TYPES =  LIGHT_TYPE_DIRECTIONAL, parent as any ptr=0) as B3D_LIGHT ptr
declare sub LightColor     (light as B3D_LIGHT ptr, red as single, green as single, blue as single)
declare sub LightConeAngles(light as B3D_LIGHT ptr, inner_ang as single, outer_ang as single)
declare sub LightRange     (light as B3D_LIGHT ptr, range as single)

' MESH
declare function LoadMesh      (file as zstring ptr, parent as B3D_ENTITY ptr=0) as B3D_MESH ptr
declare function LoadAnimMesh  (file as zstring ptr, parent as B3D_ENTITY ptr=0) as B3D_MESH ptr
declare function CreateMesh    (parent as B3D_ENTITY ptr=0) as B3D_MESH ptr
declare function CreateCone    (segments as integer=8, solid as integer=1, parent as B3D_ENTITY ptr=0) as B3D_MESH ptr
declare function CreateCylinder(segments as integer=8, solid as integer=1, parent as B3D_ENTITY ptr=0) as B3D_MESH ptr
declare function CreateCube    (parent as B3D_ENTITY ptr=0) as B3D_MESH ptr
declare function CreateQuad    (parent as B3D_ENTITY ptr=0) as B3D_MESH ptr
declare function CreatePivot   (parent as B3D_ENTITY ptr=0) as B3D_MESH ptr
declare function CreatePlane   (Divisions as integer=1, parent as B3D_ENTITY ptr=0) as B3D_MESH ptr
declare function CreateSphere  (segments as integer=8, parent as B3D_ENTITY ptr=0) as B3D_MESH ptr

declare function CopyMesh    (mesh as B3D_MESH ptr, parent as B3D_ENTITY ptr=0) as B3D_MESH ptr

declare function RepeatMesh  (mesh as B3D_MESH ptr, parent as B3D_ENTITY ptr=0) as B3D_MESH ptr
declare sub UpdateNormals(mesh as B3D_MESH ptr)
declare function MeshesIntersect(mesh as B3D_MESH ptr, mesh2 as B3D_MESH ptr) as integer
declare sub AddMesh(mesh as B3D_MESH ptr, mesh2 as B3D_MESH ptr)

declare function CountSurfaces(mesh as B3D_MESH ptr) as integer
' surface_index = 1 to CountSurfaces()
declare function GetSurface(mesh as B3D_MESH ptr, surface_index as integer) as B3D_SURFACE ptr
declare function FindSurface(mesh as B3D_MESH ptr, brush as any ptr) as B3D_SURFACE ptr

declare sub FitMesh(mesh as B3D_MESH ptr, x as single, y as single, z as single, w as single, h as single, depth as single, uniform as integer=0)

declare sub PaintMesh(mesh as B3D_MESH ptr, brush as B3D_BRUSH ptr)
declare sub ShadeMesh(mesh as B3D_MESH ptr, material as B3D_MATERIAL ptr) 

declare sub PositionMesh(mesh as B3D_MESH ptr, px as single, py as single, pz as single)
declare sub RotateMesh  (mesh as B3D_MESH ptr, pitch as single, yaw as single, roll as single)
declare sub ScaleMesh   (mesh as B3D_MESH ptr, sx as single, sy as single, sz as single)
declare sub FlipMesh    (mesh as B3D_MESH ptr)

declare function MeshWidth (mesh as B3D_MESH ptr) as single
declare function MeshHeight(mesh as B3D_MESH ptr) as single
declare function MeshDepth (mesh as B3D_MESH ptr) as single

declare function MeshCSG(mesh as B3D_MESH ptr,mesh2 as B3D_MESH ptr,method as CSG_METHODS=CSG_METHOD_UNION) as B3D_MESH ptr

' picking
declare function LinePick(px as single, py as single, pz as single, dx as single, dy as single, dz as single, radius as single=0) as B3D_ENTITY ptr
declare function PickedEntity() as B3D_ENTITY ptr
declare function PickedSurface() as B3D_SURFACE ptr
declare function PickedTime() as single
declare function PickedTriangle() as integer

declare function PickedNX() as single
declare function PickedNY() as single
declare function PickedNZ() as single

declare function PickedX() as single
declare function PickedY() as single
declare function PickedZ() as single

declare function ProjectedX() as single
declare function ProjectedY() as single
declare function ProjectedZ() as single

' SHADOW
declare function CreateShadow (parent as B3D_MESH ptr, Static_ as integer=0) as B3D_SHADOWOBJECT ptr
declare sub FreeShadow   (ShadowObject as B3D_SHADOWOBJECT ptr)

' SPRITE
declare function CreateSprite  (parent as B3D_ENTITY ptr=0) as B3D_SPRITE ptr
declare function LoadSprite    (tex_file as zstring ptr, tex_flag as TEXTURE_FLAGS =TEXTURE_FLAG_COLOR, parent as B3D_ENTITY ptr=0) as B3D_SPRITE ptr
declare sub SpriteViewMode(sprite as B3D_SPRITE ptr, ViewMode as SPRITE_VIEW_MODES = SPRITE_VIEW_MODE_FIXED)
declare sub HandleSprite  (sprite as B3D_SPRITE ptr, h_x as single, h_y as single)
declare sub ScaleSprite   (sprite as B3D_SPRITE ptr, s_x as single, s_y as single)
declare sub RotateSprite  (sprite as B3D_SPRITE ptr, ang as single)

declare function CreateVoxelSprite  (slices as integer=64, parent as B3D_ENTITY ptr=0) as B3D_VOXELSPRITE ptr
declare sub VoxelSpriteMaterial(voxelspr as B3D_VOXELSPRITE ptr, mat as B3D_MATERIAL ptr) 

' SURFACE
declare function CreateSurface   (mesh as B3D_MESH ptr, brush as B3D_BRUSH ptr=0) as B3D_SURFACE ptr
declare sub ClearSurface    (surf as B3D_SURFACE ptr, clear_verts as integer=1, clear_tris as integer=1)
declare sub PaintSurface    (surf as B3D_SURFACE ptr, brush as B3D_BRUSH ptr)
declare sub ShadeSurface    (surf as B3D_SURFACE ptr, material as B3D_MATERIAL ptr) 
declare function GetSurfaceBrush (surf as B3D_SURFACE ptr) as B3D_BRUSH ptr

declare function AddTriangle   (surf as B3D_SURFACE ptr, v0 as integer, v1 as integer, v2 as integer) as integer
declare function CountTriangles(surf as B3D_SURFACE ptr) as integer
declare function TriangleVertex(surf as B3D_SURFACE ptr, tri_no as integer, corner as integer) as integer

declare function AddVertex    (surf as B3D_SURFACE ptr, x as single, y as single, z as single, u as single=0, v as single=0, w as single=0) as integer
declare function CountVertices(surf as B3D_SURFACE ptr) as integer
declare sub VertexColor  (surf as B3D_SURFACE ptr, vid as integer, r as single, g as single, b as single, a as single=1)
declare function VertexRed    (surf as B3D_SURFACE ptr, vid as integer) as single
declare function VertexGreen  (surf as B3D_SURFACE ptr, vid as integer) as single
declare function VertexBlue   (surf as B3D_SURFACE ptr, vid as integer) as single
declare function VertexAlpha  (surf as B3D_SURFACE ptr, vid as integer) as single

declare sub VertexCoords (surf as B3D_SURFACE ptr, vid as integer, x as single, y as single, z as single)
declare function VertexX (surf as B3D_SURFACE ptr, vid as integer) as single
declare function VertexY (surf as B3D_SURFACE ptr, vid as integer) as single
declare function VertexZ (surf as B3D_SURFACE ptr, vid as integer) as single

declare sub VertexNormal (surf as B3D_SURFACE ptr, vid as integer, nx as single, ny as single, nz as single)
declare function VertexNX     (surf as B3D_SURFACE ptr, vid as integer) as single
declare function VertexNY     (surf as B3D_SURFACE ptr, vid as integer) as single
declare function VertexNZ     (surf as B3D_SURFACE ptr, vid as integer) as single

declare sub VertexTexCoords (surf as B3D_SURFACE ptr, vid as integer, u as single, v as single, w as single=0, coord_set as TEXTURE_COORD_SETS = TEXTURE_COORD_SET_FIRST)
declare function VertexU (surf as B3D_SURFACE ptr, vid as integer, coord_set as TEXTURE_COORD_SETS = TEXTURE_COORD_SET_FIRST) as single
declare function VertexV (surf as B3D_SURFACE ptr, vid as integer, coord_set as TEXTURE_COORD_SETS = TEXTURE_COORD_SET_FIRST) as single
declare function VertexW (surf as B3D_SURFACE ptr, vid as integer, coord_set as TEXTURE_COORD_SETS = TEXTURE_COORD_SET_FIRST) as single

' TERRAIN
declare function CreateTerrain (size as integer, parent as B3D_ENTITY ptr=0) as B3D_TERRAIN ptr
declare function LoadTerrain   (file as zstring ptr, parent as B3D_ENTITY ptr=0) as B3D_TERRAIN ptr
declare sub ModifyTerrain (terr as B3D_TERRAIN ptr, x as integer, z as integer, height as single)
declare function TerrainHeight (terr as B3D_TERRAIN ptr, x as integer, z as integer) as single
declare function TerrainX      (terr as B3D_TERRAIN ptr, x as single, y as single, z as single) as single
declare function TerrainY      (terr as B3D_TERRAIN ptr, x as single, y as single, z as single) as single
declare function TerrainZ      (terr as B3D_TERRAIN ptr, x as single, y as single, z as single) as single

' TEXTURE
declare function CreateTexture  (w as integer, h as integer, flags as TEXTURE_FLAGS = TEXTURE_FLAG_COLOR, frames as integer = 1) as B3D_TEXTURE ptr
declare function LoadTexture    (file as zstring ptr, flags as TEXTURE_FLAGS = TEXTURE_FLAG_COLOR) as B3D_TEXTURE ptr
declare function LoadAnimTexture(file as zstring ptr, flags as TEXTURE_FLAGS = TEXTURE_FLAG_COLOR, frame_width as integer, frame_height as integer, first_frame as integer, frame_count as integer) as B3D_TEXTURE ptr
declare sub FreeTexture    (tex as B3D_TEXTURE ptr)

declare sub PositionTexture(tex as B3D_TEXTURE ptr, u_pos as single, v_pos as single)
declare sub RotateTexture  (tex as B3D_TEXTURE ptr, ang as single)
declare sub ScaleTexture   (tex as B3D_TEXTURE ptr, u_scale as single, v_scale as single)

declare sub SetCubeFace    (tex as B3D_TEXTURE ptr, face as TEXTURE_CUBE_FACES = TEXTURE_CUBE_FACE_FORWARD)
declare sub SetCubeMode    (tex as B3D_TEXTURE ptr, mode as TEXTURE_CUBE_MODES = TEXTURE_CUBE_MODE_SPECULAR)

declare sub TextureBlend   (tex as B3D_TEXTURE ptr, BlendMode as TEXTURE_BLEND_MODES = TEXTURE_BLEND_MODE_MULTIPLY)
declare sub TextureCoords  (tex as B3D_TEXTURE ptr, coords as TEXTURE_COORD_SETS = TEXTURE_COORD_SET_FIRST)
declare function TextureWidth   (tex as B3D_TEXTURE ptr) as integer
declare function TextureHeight  (tex as B3D_TEXTURE ptr) as integer
declare function TextureName    (tex as B3D_TEXTURE ptr) as zstring ptr
declare sub TextureFilter  (match_text as zstring ptr, flags as TEXTURE_FLAGS)

' STENCIL
declare function CreateStencil() as B3D_STENCIL ptr
declare sub StencilAlpha   (stencil as B3D_STENCIL ptr, a as single)
declare sub StencilClsColor(stencil as B3D_STENCIL ptr, r as single, g as single, b as single)
declare sub StencilClsMode (stencil as B3D_STENCIL ptr, cls_depth as integer, cls_zbuffer as integer)
declare sub StencilMesh    (stencil as B3D_STENCIL ptr, mesh as B3D_MESH ptr, mode as STENCIL_MODES=STENCIL_MODE_INCR)
declare sub StencilMode    (stencil as B3D_STENCIL ptr, mode as STENCIL_MODES, operation as STENCIL_OPERATORS=STENCIL_OPERATOR_EQUAL)

' MATERIAL
declare function LoadShader   (ShaderName as zstring ptr, VertexFile   as zstring ptr, FragmentFile   as zstring ptr) as B3D_SHADER ptr
declare function CreateShader (shaderName as zstring ptr, VertexSource as zstring ptr, FragmentSource as zstring ptr) as B3D_SHADER ptr
declare sub ShaderTexture (material as B3D_MATERIAL ptr, tex as B3D_TEXTURE ptr, name_ as zstring ptr, index as integer=0) 

declare sub SetFloat    (material as B3D_SHADER ptr, name_ as zstring ptr, v1 as single) 
declare sub SetFloat2   (material as B3D_SHADER ptr, name_ as zstring ptr, v1 as single, v2 as single) 
declare sub SetFloat3   (material as B3D_SHADER ptr, name_ as zstring ptr, v1 as single, v2 as single, v3 as single) 
declare sub SetFloat4   (material as B3D_SHADER ptr, name_ as zstring ptr, v1 as single, v2 as single, v3 as single, v4 as single) 

declare sub UseFloat    (material as B3D_SHADER ptr, name_ as zstring ptr, byref v1 as single) 
declare sub UseFloat2   (material as B3D_SHADER ptr, name_ as zstring ptr, byref v1 as single, byref v2 as single) 
declare sub UseFloat3   (material as B3D_SHADER ptr, name_ as zstring ptr, byref v1 as single, byref v2 as single, byref v3 as single) 
declare sub UseFloat4   (material as B3D_SHADER ptr, name_ as zstring ptr, byref v1 as single, byref v2 as single, byref v3 as single, byref v4 as single) 

declare sub SetInteger  (material as B3D_SHADER ptr, name_ as zstring ptr, v1 as single) 
declare sub SetInteger2 (material as B3D_SHADER ptr, name_ as zstring ptr, v1 as single, v2 as single) 
declare sub SetInteger3 (material as B3D_SHADER ptr, name_ as zstring ptr, v1 as single, v2 as single, v3 as single) 
declare sub SetInteger4 (material as B3D_SHADER ptr, name_ as zstring ptr, v1 as single, v2 as single, v3 as single, v4 as single) 

declare sub UseInteger  (material as B3D_SHADER ptr, name_ as zstring ptr, byref v1 as single) 
declare sub UseInteger2 (material as B3D_SHADER ptr, name_ as zstring ptr, byref v1 as single, byref v2 as single) 
declare sub UseInteger3 (material as B3D_SHADER ptr, name_ as zstring ptr, byref v1 as single, byref v2 as single, byref v3 as single) 
declare sub UseInteger4 (material as B3D_SHADER ptr, name_ as zstring ptr, byref v1 as single, byref v2 as single, byref v3 as single, byref v4 as single) 

declare sub UseSurface (material as B3D_SHADER ptr, name_ as zstring ptr, surf as B3D_SURFACE ptr, vbo as integer) 
declare sub UseMatrix  (material as B3D_SHADER ptr, name_ as zstring ptr, mode as integer) 

end extern

#endif ' __OPENB3D_BI__

